@@ -30,18 +30,18 @@
 
 DO_COMMAND(do_alias)
 {
-	char *left, *right, *rank;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], rank[BUFFER_SIZE];
 	struct listroot *root;
 
 	root = ses->list[LIST_ALIAS];
 
-	arg = get_arg_in_braces(arg, &left,  FALSE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-	arg = get_arg_in_braces(arg, &rank,  FALSE);
+	arg = get_arg_in_braces(arg, left,  FALSE);
+	arg = get_arg_in_braces(arg, right, TRUE);
+	arg = get_arg_in_braces(arg, rank,  FALSE);
 
 	if (*rank == 0)
 	{
-		rank = string_alloc("5");
+		strcpy(rank, "5");
 	}
 
 	if (*left == 0)
@@ -59,7 +59,7 @@ DO_COMMAND(do_alias)
 	{
 		updatenode_list(ses, left, right, rank, LIST_ALIAS);
 
-		show_message(ses, LIST_ALIAS, "#OK. {%s} NOW ALIASSES {%s} @ {%s}.", left, right, rank);
+		show_message(ses, LIST_ALIAS, "#OK. {%s} NOW ALIASES {%s} @ {%s}.", left, right, rank);
 	}
 	return ses;
 }
@@ -74,7 +74,7 @@ DO_COMMAND(do_unalias)
 
 int alias_regexp(char *exp, char *str)
 {
-	short cnt;
+	short cnt, arg;
 
 	while (*exp)
 	{
@@ -94,29 +94,27 @@ int alias_regexp(char *exp, char *str)
 		switch (exp[0])
 		{
 			case '%':
-				if (exp[1] >= '0' && exp[1] <= '9')
+				if (isdigit(exp[1]))
 				{
-					if (exp[2] == 0)
+					arg = isdigit(exp[2]) ? (exp[1] - '0') * 10 + exp[2] - '0' : exp[1] - '0';
+
+					if (exp[isdigit(exp[2]) ? 3 : 2] == 0)
 					{
-						gtd->vars[exp[1] - '0'] = string_realloc(gtd->vars[exp[1] - '0'], str);
+						RESTRING(gtd->vars[arg], str);
 
 						return TRUE;
 					}
 
-					for (cnt = 0 ; cnt < 1000 ; cnt++)
+					for (cnt = 0 ; str[cnt] ; cnt++)
 					{
-						if (alias_regexp(exp + 2, &str[cnt]))
+						if (alias_regexp(exp + (isdigit(exp[2]) ? 3 : 2), &str[cnt]))
 						{
-							gtd->vars[exp[1] - '0'] = string_realloc(gtd->vars[exp[1] - '0'], str);
-							gtd->vars[exp[1] - '0'][cnt] = 0;
+							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt, str);
 
 							return TRUE;
 						}
-						if (str[cnt] == 0)
-						{
-							return FALSE;
-						}
 					}
+					return FALSE;
 				}
 				break;
 
@@ -143,40 +141,46 @@ int alias_regexp(char *exp, char *str)
 	return FALSE;
 }
 
-int check_all_aliases(struct session *ses, char *str1, char *str2, char **command)
+int check_all_aliases(struct session *ses, char *input)
 {
 	struct listnode *node;
 	struct listroot *root;
-	char *arg;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], *arg;
 	int i;
 
 	root = ses->list[LIST_ALIAS];
 
-	gtd->vars[0] = string_alloc(str2);
+	arg = get_arg_stop_spaces(input, left);
 
-	arg = str2;
+	RESTRING(gtd->vars[0], arg);
 
-	for (i = 1 ; i < 10 ; i++)
+	for (i = 1 ; i < 100 ; i++)
 	{
-		arg = get_arg_in_braces(arg, &gtd->vars[i], FALSE);
-	}
+		arg = get_arg_in_braces(arg, right, FALSE);
 
-	arg = stringf_alloc("%s%s%s", str1, *str2 ? " " : "", str2);
+		RESTRING(gtd->vars[i], right);
+	}
 
 	for (node = root->update = root->f_node ; node ; node = root->update)
 	{
 		root->update = node->next;
 
-		if (alias_regexp(node->left, arg))
+		if (alias_regexp(node->left, input))
 		{
-			substitute(ses, node->right, command, SUB_ARG);
+			substitute(ses, node->right, right, SUB_ARG);
 
-			if (!strcmp(node->right, *command) && *str2)
+			if (!strcmp(node->right, right) && *arg)
 			{
-				*command = stringf_alloc("%s %s", *command, str2);
+				sprintf(input, "%s %s", right, arg);
+			}
+			else
+			{
+				sprintf(input, "%s", right);
 			}
 
-			show_debug(ses, LIST_ALIAS, "#ALIAS DEBUG: %s : %s", str1, str2);
+			show_debug(ses, LIST_ALIAS, "#ALIAS DEBUG: %s: %s", left, input);
+
+			DEL_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
 
 			return TRUE;
 		}
