@@ -37,11 +37,11 @@ DO_COMMAND(do_read)
 {
 	FILE *fp;
 	struct stat filedata;
-	char *bufi, *bufo, *filename, temp[BUFFER_SIZE], *pti, *pto;
+	char *bufi, *bufo, filename[BUFFER_SIZE], temp[BUFFER_SIZE], *pti, *pto;
 	int lvl, cnt, com, lnc, fix, ok;
 	int counter[LIST_MAX];
 
-	get_arg_in_braces(arg, &filename, TRUE);
+	get_arg_in_braces(arg, filename, TRUE);
 
 	if ((fp = fopen(filename, "r")) == NULL)
 	{
@@ -273,7 +273,7 @@ DO_COMMAND(do_read)
 
 	sprintf(temp, "{TINTIN CHAR} {%c}", bufo[0]);
 
-	SET_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+	gtd->quiet++;
 
 	do_configure(ses, temp);
 
@@ -292,7 +292,7 @@ DO_COMMAND(do_read)
 
 		if (strlen(bufi) >= BUFFER_SIZE)
 		{
-			DEL_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+			gtd->quiet--;
 
 			bufi[20] = 0;
 
@@ -308,13 +308,13 @@ DO_COMMAND(do_read)
 
 		if (bufi[0])
 		{
-			ses = parse_input(ses, bufi);
+			ses = pre_parse_input(ses, bufi, SUB_NONE);
 		}
 		pto = bufi;
 		pti++;
 	}
 
-	DEL_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+	gtd->quiet--;
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_VERBOSE))
 	{
@@ -350,11 +350,11 @@ DO_COMMAND(do_read)
 DO_COMMAND(do_write)
 {
 	FILE *file;
-	char temp[STRING_SIZE], *filename;
+	char temp[STRING_SIZE], filename[BUFFER_SIZE];
 	struct listnode *node;
 	int cnt;
 
-	get_arg_in_braces(arg, &filename, TRUE);
+	get_arg_in_braces(arg, filename, TRUE);
 
 	if (*filename == 0 || (file = fopen(filename, "w")) == NULL)
 	{
@@ -407,71 +407,3 @@ void prepare_for_write(int list, struct listnode *node, char *result)
 	}
 }
 
-
-DO_COMMAND(do_textin)
-{
-	FILE *fp;
-	char *filename, buffer[BUFFER_SIZE], *cptr;
-
-	get_arg_in_braces(arg, &filename, TRUE);
-
-	if ((fp = fopen(filename, "r")) == NULL)
-	{
-		tintin_printf(ses, "#ERROR: #TEXTIN {%s} - FILE NOT FOUND.", filename);
-		return ses;
-	}
-
-	while (fgets(buffer, sizeof(buffer), fp))
-	{
-		for (cptr = buffer ; *cptr && *cptr != '\n' ; cptr++)
-		{
-			;
-		}
-		*cptr++ = '\r';
-		*cptr++ = '\n';
-		*cptr++ = '\0';
-
-		write_line_mud(buffer, ses);
-	}
-	fclose(fp);
-
-	show_message(ses, -1, "#OK. FILE READ.");
-
-	return ses;
-}
-
-
-DO_COMMAND(do_scan)
-{
-	FILE *fp;
-	char *filename;
-
-	get_arg_in_braces(arg, &filename, TRUE);
-
-	if ((fp = fopen(filename, "r")) == NULL)
-	{
-		tintin_printf(ses, "#ERROR: #SCAN {%s} - FILE NOT FOUND.", filename);
-		return ses;
-	}
-
-	SET_BIT(ses->flags, SES_FLAG_SCAN);
-
-	if (STRING_SIZE > gtd->mud_output_max)
-	{
-		gtd->mud_output_max  = STRING_SIZE;
-		gtd->mud_output_buf  = realloc(gtd->mud_output_buf, gtd->mud_output_max);
-	}
-
-	while (fgets(gtd->mud_output_buf, STRING_SIZE, fp))
-	{
-		readmud(ses);
-	}
-
-	DEL_BIT(ses->flags, SES_FLAG_SCAN);
-
-	show_message(ses, -1, "#OK. FILE READ.", filename);
-
-	fclose(fp);
-
-	return ses;
-}
