@@ -128,13 +128,13 @@ struct help_type help_table[] =
 		"         #chat {name}       {name}             Sets your chat name.\n"
 		"         #chat {message}    {buddy|all} {text} Sends a chat message\n"
 		"\n"
-		"         #chat {accept}     {buddy} {boosted}  Accept a file transfer\n"
+		"         #chat {accept}     {buddy}            Accept a file transfer\n"
 		"         #chat {call}       {address} {port}   Connect to a buddy\n"
 		"         #chat {cancel}     {buddy}            Cancel a file transfer\n"
 		"         #chat {color}      {color names}      Set the default color\n"
 		"         #chat {decline}    {buddy}            Decline a file transfer\n"
 		"         #chat {dnd}                           Decline new connections\n"
-		"         #chat {download}   {buddy}            Set your download dir\n"
+		"         #chat {download}   {directory}        Set your download directory\n"
 		"         #chat {emote}      {buddy|all} {text} Send an emote message\n"
 		"         #chat {forward}    {buddy}            Forward all chat messages\n"
 		"         #chat {forwardall} {buddy}            Forward all session output\n"
@@ -219,6 +219,7 @@ struct help_type help_table[] =
 		"         Config options which aren't listed by default:\n"
 		"\n"
 		"         #CONFIG {CONVERT META} {ON|OFF} Shows color codes and key bindings.\n"
+		"         #CONFIG {COLOR PATCH}  {ON|OFF} Fixes color code usage on some muds.\n"
 		"         #CONFIG {DEBUG TELNET} {ON|OFF} Shows telnet negotiations y/n.\n"
 		"         #CONFIG {LOG LEVEL}  {LOW|HIGH} LOW logs mud output before triggers\n"
 	},
@@ -329,7 +330,7 @@ struct help_type help_table[] =
 	},
 	{
 		"FORMAT",
-		"<178>Command<078>: #format <178>{<078>variable<178>}<078> <178>{<078>format<178>}<078> <178>{<078>argument1 argument2 etc<178>}<078>\n"
+		"<178>Command<078>: #format <178>{<078>variable<178>}<078> <178>{<078>format<178>}<078> <178>{{<078>argument1<178>} {<078>argument2<178>} {<078>etc<178>}}<078>\n"
 		"\n"
 		"         Allows you to store a string into a variable in the exact same way\n"
 		"         C's sprintf works with a few enhancements and limitations such as\n"
@@ -349,15 +350,16 @@ struct help_type help_table[] =
 		"         #format {test} {%r}   {string}  reverse text, hiya = ayih\n"
 		"         #format {test} {%t}     {time}  print the current military time\n"
 		"         #format {test} {%u}   {string}  uppercase text\n"
-		"         #format {test} {%C}   {colums}  store the screen width in {colums}\n"
+		"         #format {test} {%w}   {string}  wordwrap text\n"
+		"         #format {cols} {%C}         {}  store the screen width in {cols}\n"
 		"         #format {test} {%D}     {time}  print the current day\n"
 		"         #format {test} {%G}   {number}  perform thousand grouping on {number}\n"
 		"         #format {test} {%L}   {string}  store the string length in {test}\n"
 		"         #format {test} {%M}     {time}  print the current month\n"
-		"         #format {test} {%R}     {rows}  store the screen height in {rows}\n"
-		"         #format {time} {%T}             store the epoch time in {time}\n"
-		"         #format {time} {%U}             store the micro epoch time in {time}\n"
-		"         #format {time} {%Y}     {time}  print the current year\n"
+		"         #format {rows} {%R}         {}  store the screen height in {rows}\n"
+		"         #format {time} {%T}         {}  store the epoch time in {time}\n"
+		"         #format {time} {%U}         {}  store the micro epoch time in {time}\n"
+		"         #format {test} {%Y}     {time}  print the current year\n"
 	},
 	{
 		"FUNCTION",
@@ -399,7 +401,11 @@ struct help_type help_table[] =
 		"GAGLINE",
 		"<178>Command<078>: #gagline\n"
 		"\n"
-		"         When called from within an action the trigger string is not displayed.\n"
+		"         The gagline command will stop the next line from being displayed.\n"
+		"         If used in an action it will work in a similar fashion as the gag\n"
+		"         command. If you use gagline before a showme in an action the showme\n"
+		"         will be gagged, rather than the line triggering the action. To avoid\n"
+		"         this place the gagline command add the end of the action."
 	},
 	{
 		"GREP",
@@ -620,6 +626,10 @@ struct help_type help_table[] =
 		"                  Keep in mind the exit command is only used when using\n"
 		"                  '#map find' and '#map walk'.\n"
 		"\n"
+		"         #map explore <direction>: Follows the direction until a dead end or an\n"
+		"                  intersection is found. The route is stored in #path and can\n"
+		"                  subsequently be used with #walk\n"
+		"\n"
 		"         #map find <room name>: Searches for the given room name. If found the\n"
 		"                  shortest path from your current location to the target\n"
 		"                  location is calculated. The route is stored in #path and can\n"
@@ -680,6 +690,10 @@ struct help_type help_table[] =
 		"                  automatically leave the map. Useful when set at the entrance\n"
 		"                  of an unmappable maze.\n"
 		"\n"
+		"         #map travel <direction> <delay>: Follows the direction until a dead end or an\n"
+		"                  intersection is found. Use braces around the direction if you use\n"
+		"                  the delay, which will add the given delay between movements.\n"
+		"\n"
 		"         #map undo: Will delete the room you are in and move you out.\n"
 		"                  Useful if you walked into a non existant direction.\n"
 		"\n"
@@ -773,7 +787,7 @@ struct help_type help_table[] =
 	},
 	{
 		"PATH",
-		"<178>Command<078>: #path <178>{<078>del|end|ins|load|map|new|save|walk<178>}<078> <178>{<078>argument<178>}<078>\n"
+		"<178>Command<078>: #path <178>{<078>del|end|ins|load|map|new|run|save|walk<178>}<078> <178>{<078>argument<178>}<078>\n"
 		"\n"
 		"         #path del:  Will delete the last move of the path.\n"
 		"         #path end:  Movement commands are no longer added to the path.\n"
@@ -781,6 +795,8 @@ struct help_type help_table[] =
 		"         #path load: Load the given alias as the new path.\n"
 		"         #path map:  Display the so far created path.\n"
 		"         #path new:  Movement commands will be added to the path.\n"
+		"         #path run:  Enter all the steps of the current path, with\n"
+		"                     an optional given delay.\n"
 		"         #path save: Save the created path to an alias. You must specify whether\n"
 		"                     you want to save the path forward or backward.\n"
 		"         #path walk: Take one step forward of the current path. You can walk\n"
@@ -837,6 +853,16 @@ struct help_type help_table[] =
 		"\n"
 		"         You can comment out triggers using /* text */\n"
 	},
+	{
+		"REGEXP",
+		"<178>Command<078>: #regexp <178>{<078>string<178>}<078> <178>{<078>expression<178>}<078> <178>{<078>true<178>}<078> <178>{<078>false<178>}<078>\n"
+		"\n"
+		"         Compares the string to the given regular expression.\n"
+		"\n"
+		"         Variables are stored in &1 to &9 with &0 holding the matched substring.\n"
+		"\n"
+		"<178>Example<078>: #regexp {bli bla blo} {bli (.*) blo} {#showme &1}\n"
+	},
 	{
 		"REPLACESTRING",
 		"<178>Command<078>: #replacestring <178>{<078>variable<178>}<078> <178>{<078>oldtext<178>}<078> <178>{<078>newtext<178>}<078>\n"
@@ -852,6 +878,19 @@ struct help_type help_table[] =
 		"         string being executed. In a #function you can use #return with an\n"
 		"         argument to both break out of the function and set the result variable.\n"
 	},
+	{
+		"RUN",
+		"<178>Command<078>: #run <178>{<078>session name<178>}<078> <178>{<078>shell command<178>}<078>\n"
+		"\n"
+		"         The run command works much like the system command except that it\n"
+		"         runs the command in a pseudo terminal. The run command also creates\n"
+		"         a session that treats the given shell command as a mud server. This\n"
+		"         allows you to run ssh, as well as any other shell application, with\n"
+		"         full tintin scripting capabilities\n"
+		"\n"
+		"<178>Example<078>: #run {somewhere} {ssh someone@somewhere.com}\n"
+		"<178>Example<078>: #run {something} {tail -f chats.log}\n"
+	},
 	{
 		"SCAN",
 		"<178>Command<078>: #scan <178>{<078>filename<178>}<078>\n"
@@ -867,6 +906,22 @@ struct help_type help_table[] =
 		"         This is useful to convert ansi color file to html or viewing log file.\n"
 		
 	},
+	{
+		"SCRIPT",
+		"<178>Command<078>: #script <178>{<078>shell command<178>}\n"
+		"\n"
+		"         The script command works much like the system command except that it\n"
+		"         treats the generated echos as commands.\n"
+		"\n"
+		"         This is useful for running php, perl, ruby, and python scripts. You\n"
+		"         can run these scrips either from file or from within tintin if the\n"
+		"         scripting language allows this.\n"
+		"\n"
+		"<178>Example<078>: #script {ruby -e 'print \"#showme hello world\"'}\n"
+		"<178>Example<078>: #script {python -c 'print \"#showme hello world\"'}\n"
+		"<178>Example<078>: #script {php -r 'echo \"#showme hello world\"'}\n"
+	},
+
 	{
 		"SEND",
 		"<178>Command<078>: #send <178>{<078>text<178>}<078>\n"
@@ -1101,10 +1156,10 @@ struct help_type help_table[] =
 
 DO_COMMAND(do_help)
 {
-	char *left, add[BUFFER_SIZE], *ptf, *pto;
+	char left[BUFFER_SIZE], add[BUFFER_SIZE], buf[BUFFER_SIZE], *ptf, *pto;
 	int cnt, found;
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
+	arg = get_arg_in_braces(arg, left, TRUE);
 
 	if (*left == 0)
 	{
@@ -1131,12 +1186,14 @@ DO_COMMAND(do_help)
 		{
 			if (is_abbrev(left, help_table[cnt].name) || atoi(left) == cnt + 1 || regexp(left, help_table[cnt].name, FALSE))
 			{
-				substitute(ses, help_table[cnt].text, &pto, SUB_COL);
+				substitute(ses, help_table[cnt].text, buf, SUB_COL);
 
 				tintin_header(ses, " %s ", help_table[cnt].name);
 
 				tintin_puts2(ses, "");
 
+				pto = buf;
+
 				while (*pto)
 				{
 					ptf = strchr(pto, '\n');
@@ -1154,6 +1211,11 @@ DO_COMMAND(do_help)
 				tintin_puts2(ses, "");
 
 				found = TRUE;
+
+				if (is_abbrev(left, help_table[cnt].name))
+				{
+					break;
+				}
 			}
 		}
 		if (found == FALSE)
