@@ -50,7 +50,7 @@ void process_input(void)
 
 	if (gtd->chat && gtd->chat->paste_time)
 	{
-		chat_paste(line);
+		chat_paste(line, NULL);
 
 		return;
 	}
@@ -76,7 +76,7 @@ void process_input(void)
 
 	SET_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
 
-	parse_input(gtd->ses, line);
+	pre_parse_input(gtd->ses, line, SUB_NONE);
 
 	DEL_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
 
@@ -105,44 +105,49 @@ void read_line(char *line)
 	{
 		strcat(gtd->macro_buf, buffer);
 	}
-	match = 0;
-	root  = gtd->ses->list[LIST_MACRO];
 
-	for (node = root->f_node ; node ; node = root->update)
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_CONVERTMETA))
 	{
-		root->update = node->next;
+		match = 0;
+		root  = gtd->ses->list[LIST_MACRO];
 
-		if (!strcmp(gtd->macro_buf, node->pr))
+		for (node = root->f_node ; node ; node = root->update)
 		{
-			parse_input(gtd->ses, node->right);
-			gtd->macro_buf[0] = 0;
+			root->update = node->next;
 
+			if (!strcmp(gtd->macro_buf, node->pr))
+			{
+				pre_parse_input(gtd->ses, node->right, SUB_NONE);
+
+				gtd->macro_buf[0] = 0;
+
+				return;
+			}
+			else if (!strncmp(gtd->macro_buf, node->pr, strlen(gtd->macro_buf)))
+			{
+				match = 1;
+			}
+		}
+
+		for (cnt = 0 ; *cursor_table[cnt].code != 0 ; cnt++)
+		{
+			if (!strcmp(gtd->macro_buf, cursor_table[cnt].code))
+			{
+				cursor_table[cnt].fun("");
+				gtd->macro_buf[0] = 0;
+
+				return;
+			}
+			else if (!strncmp(gtd->macro_buf, cursor_table[cnt].code, strlen(gtd->macro_buf)))
+			{
+				match = 1;
+			}
+		}
+
+		if (match)
+		{
 			return;
 		}
-		else if (!strncmp(gtd->macro_buf, node->pr, strlen(gtd->macro_buf)))
-		{
-			match = 1;
-		}
-	}
-
-	for (cnt = 0 ; *cursor_table[cnt].code != 0 ; cnt++)
-	{
-		if (!strcmp(gtd->macro_buf, cursor_table[cnt].code))
-		{
-			cursor_table[cnt].fun("");
-			gtd->macro_buf[0] = 0;
-
-			return;
-		}
-		else if (!strncmp(gtd->macro_buf, cursor_table[cnt].code, strlen(gtd->macro_buf)))
-		{
-			match = 1;
-		}
-	}
-
-	if (match)
-	{
-		return;
 	}
 
 	if (gtd->macro_buf[0] == ESCAPE)
@@ -168,6 +173,7 @@ void read_line(char *line)
 				gtd->input_pos    = 0;
 				gtd->input_buf[0] = 0;
 				gtd->macro_buf[0] = 0;
+				gtd->input_tmp[0] = 0;
 
 				if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
 				{
@@ -185,20 +191,32 @@ void read_line(char *line)
 				if (HAS_BIT(gtd->flags, TINTIN_FLAG_INSERTINPUT) && gtd->input_len != gtd->input_cur)
 				{
 					gtd->input_buf[gtd->input_cur] = gtd->macro_buf[cnt];
+
 					gtd->input_cur++;
 					gtd->input_pos++;
+
 					input_printf("%c", gtd->macro_buf[cnt]);
 				}
 				else
 				{
 					ins_sprintf(&gtd->input_buf[gtd->input_cur], "%c", gtd->macro_buf[cnt]);
+
 					gtd->input_len++;
 					gtd->input_cur++;
 					gtd->input_pos++;
-					input_printf("\033[1@%c", gtd->macro_buf[cnt]);
+
+					if (gtd->input_len != gtd->input_cur)
+					{
+						input_printf("\033[1@%c", gtd->macro_buf[cnt]);
+					}
+					else
+					{
+						input_printf("%c", gtd->macro_buf[cnt]);
+					}
 				}
 
 				gtd->macro_buf[0] = 0;
+				gtd->input_tmp[0] = 0;
 				gtd->input_buf[gtd->input_len] = 0;
 
 				cursor_check_line("");
@@ -239,59 +257,70 @@ void read_key(char *line)
 		strcat(gtd->macro_buf, buffer);
 	}
 
-	match = 0;
-	root  = gtd->ses->list[LIST_MACRO];
-
-	for (node = root->f_node ; node ; node = root->update)
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_CONVERTMETA))
 	{
-		root->update = node->next;
+		match = 0;
+	
+		root  = gtd->ses->list[LIST_MACRO];
 
-		if (!strcmp(gtd->macro_buf, node->pr))
+		for (node = root->f_node ; node ; node = root->update)
+		{
+			root->update = node->next;
+
+			if (!strcmp(gtd->macro_buf, node->pr))
+			{
+				pre_parse_input(gtd->ses, buffer, SUB_NONE);
+
+				gtd->macro_buf[0] = 0;
+				return;
+			}
+			else if (!strncmp(gtd->macro_buf, node->pr, strlen(gtd->macro_buf)))
+			{
+				match = 1;
+			}
+		}
+
+		if (match)
 		{
-			parse_input(gtd->ses, node->right);
-			gtd->macro_buf[0] = 0;
 			return;
 		}
-		else if (!strncmp(gtd->macro_buf, node->pr, strlen(gtd->macro_buf)))
-		{
-			match = 1;
-		}
-	}
-
-	if (match)
-	{
-		return;
 	}
 
 	for (cnt = 0 ; gtd->macro_buf[cnt] ; cnt++)
 	{
 		switch (gtd->macro_buf[cnt])
 		{
-			case '\n':
 			case '\r':
+			case '\n':
 				gtd->input_buf[0] = 0;
 				gtd->macro_buf[0] = 0;
-				echo_command(gtd->ses, "", FALSE);
-				socket_printf(gtd->ses, 2, "%c%c", '\r', '\0');
+				socket_printf(gtd->ses, 1, "%c", '\r'); 
 				break;
 
 			default:
 				if (gtd->macro_buf[cnt] == gtd->tintin_char && gtd->input_buf[0] == 0)
 				{
-					printf("\033[1@%c", gtd->macro_buf[cnt]);
-
+					if (gtd->input_len != gtd->input_cur)
+					{
+						printf("\033[1@%c", gtd->macro_buf[cnt]);
+					}
+					else
+					{
+						printf("%c", gtd->macro_buf[cnt]);
+					}
 					gtd->input_buf[0] = gtd->tintin_char;
+					gtd->input_buf[1] = 0;
+					gtd->macro_buf[0] = 0;
 					gtd->input_len = 1;
 					gtd->input_cur = 1;
 					gtd->input_pos = 1;
-					gtd->macro_buf[0] = 0;
-					gtd->input_buf[1] = 0;
 				}
 				else
 				{
 					socket_printf(gtd->ses, 1, "%c", gtd->macro_buf[cnt]);
 					gtd->input_buf[0] = 127;
 					gtd->macro_buf[0] = 0;
+					gtd->input_len = 0;
 				}
 				break;
 		}
@@ -393,11 +422,11 @@ void unconvert_meta(char *input, char *output)
 
 void echo_command(struct session *ses, char *line, int newline)
 {
-	char buffer[STRING_SIZE];
+	char buffer[STRING_SIZE], result[STRING_SIZE];
 
-	if (IS_SPLIT(ses) && ses->more_output[0] == 0)
+	if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 	{
-		sprintf(buffer, "\033[0m%s", line);
+		sprintf(buffer, "\033[0;37m%s", line);
 	}
 	else
 	{
@@ -412,38 +441,57 @@ void echo_command(struct session *ses, char *line, int newline)
 		}
 		ses->check_output = 0;
 
-		SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
+		if (HAS_BIT(ses->flags, SES_FLAG_ECHOCOMMAND))
+		{
+			SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
 
-		tintin_printf2(ses, "%s", buffer);
+			if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
+			{
+				tintin_printf2(ses, "%s%s", ses->more_output, buffer);
+			}
+			else
+			{
+				tintin_printf2(ses, "%s%s", buffer);
+			}
 
-		DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
-
-		add_line_buffer(ses, buffer, FALSE);
+			DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
+		}
+		add_line_buffer(ses, buffer, -1);
 
 		return;
 	}
 
-	if (IS_SPLIT(ses))
+	DEL_BIT(ses->telopts, TELOPT_FLAG_PROMPT);
+
+	if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 	{
-		if (!HAS_BIT(ses->flags, SES_FLAG_ECHOCOMMAND) && ses->more_output[0] == 0)
+		if (!HAS_BIT(ses->flags, SES_FLAG_ECHOCOMMAND))
 		{
-			return;
-		}
-		SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
-
-		if (ses->more_output[0])
-		{
-			ses->check_output = 0;
-
-			tintin_printf2(ses, "%s%s", ses->more_output, buffer);
+			sprintf(result, "\033[0;37m");
 		}
 		else
 		{
-			tintin_printf2(ses, "%s", buffer);
+			if (ses->more_output[0])
+			{
+				sprintf(result, "%s\033[0;37m%s", ses->more_output, line);
+			}
+			else
+			{
+				sprintf(result, "\033[0;37m%s", line);
+			}
 		}
+		add_line_buffer(ses, buffer, -1);
+
+		SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
+
+		tintin_printf2(ses, "%s", result);
+
 		DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
 	}
-	add_line_buffer(ses, buffer, -1);
+	else
+	{
+		add_line_buffer(ses, buffer, -1);
+	}
 }
 
 void input_printf(char *format, ...)
