@@ -143,8 +143,6 @@ int main(int argc, char **argv)
 		SOCKSinit(argv[0]);
 	#endif
 
-	push_call("main(%p,%p)",argc,argv);
-
 	init_tintin();
 
 	if (signal(SIGTERM, trap_handler) == BADSIG)
@@ -200,14 +198,14 @@ int main(int argc, char **argv)
 
 	if (argc > 1)
 	{
-		char c;
+		int c;
 
 		while ((c = getopt(argc, argv, "e: h r: t: v")) != EOF)
 		{
 			switch (c)
 			{
 				case 'e':
-					gtd->ses = parse_input(gtd->ses, optarg);
+					gtd->ses = pre_parse_input(gtd->ses, optarg, SUB_NONE);
 					break;
 
 				case 'h':
@@ -242,7 +240,6 @@ int main(int argc, char **argv)
 
 	mainloop();
 
-	pop_call();
 	return 0;
 }
 
@@ -263,6 +260,7 @@ void init_tintin(void)
 	gts->host           = strdup("");
 	gts->port           = strdup("");
 	gts->telopts        = TELOPT_FLAG_ECHO;
+	gts->socket         = 1;
 
 	gtd                 = calloc(1, sizeof(struct tintin_data));
 
@@ -278,6 +276,12 @@ void init_tintin(void)
 	gtd->mud_output_max = 64;
 	gtd->mud_output_buf = calloc(1, gtd->mud_output_max);
 
+	for (cnt = 0 ; cnt < 99 ; cnt++)
+	{
+		gtd->vars[cnt] = strdup("");
+		gtd->cmds[cnt] = strdup("");
+	}
+
 	for (ref = 0 ; ref < 26 ; ref++)
 	{
 		for (cnt = 0 ; *command_table[cnt].name != 0 ; cnt++)
