@@ -57,7 +57,7 @@ unsigned char	map_palet0[]	= { 42, 35, 35, 43, 35,124, 43, 43, 35, 43, 45, 43, 4
 
 unsigned char	map_palet1[]	= {126,247,247,109,247,120,108,116,247,106,113,118,107,117,119,110, 88};
 
-#define CHECK_MAP(void)                                                                                            \
+#define CHECK_MAP()                                                                                            \
 {                                                                                                                  \
 	if (ses->map == NULL)                                                                                         \
 	{                                                                                                             \
@@ -67,7 +67,7 @@ unsigned char	map_palet1[]	= {126,247,247,109,247,120,108,116,247,106,113,118,10
 	}                                                                                                             \
 }
 
-#define CHECK_INSIDE(void)                                                                                         \
+#define CHECK_INSIDE()                                                                                         \
 {                                                                                                                  \
 	if (ses->map == NULL)                                                                                         \
 	{                                                                                                             \
@@ -86,9 +86,9 @@ unsigned char	map_palet1[]	= {126,247,247,109,247,120,108,116,247,106,113,118,10
 DO_COMMAND(do_map)
 {
 	int cnt;
-	char *cmd;
+	char cmd[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, &cmd, FALSE);
+	arg = get_arg_in_braces(arg, cmd,   0);
 
 	if (*cmd == 0)
 	{
@@ -99,6 +99,7 @@ DO_COMMAND(do_map)
 		tintin_printf2(ses, "#map destroy                           (destroys the map)");
 		tintin_printf2(ses, "#map dig      <direction>              (creates a new room)");
 		tintin_printf2(ses, "#map exit     <direction> <command>    (sets the exit command)");
+		tintin_printf2(ses, "#map explore  <direction>              (saves path to #path)");
 		tintin_printf2(ses, "#map info                              (info on map and current room)");
 		tintin_printf2(ses, "#map insert   <direction>              (insert a new room)");
 		tintin_printf2(ses, "#map find     <room name>              (saves path to #path)");
@@ -113,6 +114,7 @@ DO_COMMAND(do_map)
 		tintin_printf2(ses, "#map read     <filename>               (load your map from a file)");
 		tintin_printf2(ses, "#map roomflag <room flag>              (set room based flags)");
 		tintin_printf2(ses, "#map set      <option>     <value>     (set various values)");
+		tintin_printf2(ses, "#map travel   <direction>              (walk to explored direction)");
 		tintin_printf2(ses, "#map undo                              (undo last move)");
 		tintin_printf2(ses, "#map unlink   <direction>              (deletes an exit)");
 		tintin_printf2(ses, "#map walk     <room name>              (walk to given room)");
@@ -124,9 +126,12 @@ DO_COMMAND(do_map)
 	{
 		for (cnt = 0 ; *map_table[cnt].name ; cnt++)
 		{
-			if (is_abbrev(cmd, map_table[cnt].name))
+			if (is_abbrev(left, map_table[cnt].name))
 			{
-				map_table[cnt].map (ses, arg);
+				arg = get_arg_in_braces(arg, left,  map_table[cnt].lval);
+				arg = get_arg_in_braces(arg, right, map_table[cnt].rval);
+
+				map_table[cnt].map (ses, left, right);
 				break;
 			}
 		}
@@ -146,12 +151,8 @@ DO_COMMAND(do_map)
 
 DO_MAP(map_color)
 {
-	char *left;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-
 	free(ses->map->room_list[ses->map->in_room]->color);
 	ses->map->room_list[ses->map->in_room]->color = strdup(left);
 
@@ -168,13 +169,10 @@ DO_MAP(map_create)
 DO_MAP(map_delete)
 {
 	int room;
-	char *left;
 	struct exit_data *exit;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-
 	exit = find_exit(ses, ses->map->in_room, left);
 
 	if (exit == NULL)
@@ -206,12 +204,9 @@ DO_MAP(map_dig)
 	char temp[BUFFER_SIZE];
 	struct exit_data *exit;
 	struct listnode *node;
-	char *left;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	for (room = 1 ; room < MAX_ROOM ; room++)
 	{
 		if (ses->map->room_list[room] == NULL)
@@ -262,13 +257,9 @@ DO_MAP(map_exit)
 {
 	char temp[BUFFER_SIZE];
 	struct exit_data *exit;
-	char *left, *right;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-
 	exit = find_exit(ses, ses->map->in_room, left);
 
 	if (exit == NULL)
@@ -287,25 +278,25 @@ DO_MAP(map_exit)
 	tintin_printf2(ses, "#MAP: Exit {%s} set to {%s}.", left, right);
 }
 
-DO_MAP(map_find)
+DO_MAP(map_explore)
 {
-	char *left;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
+	explore_path(ses, FALSE, left, "");
+}
+
+DO_MAP(map_find)
+{
+	CHECK_INSIDE();
 
 	shortest_path(ses, FALSE, left, "");
 }
 
 DO_MAP(map_flag)
 {
-	char *left;
-
 	CHECK_MAP();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if (*left)
 	{
 		if (is_abbrev(left, "static"))
@@ -349,13 +340,8 @@ DO_MAP(map_flag)
 
 DO_MAP(map_get)
 {
-	char *left, *right;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-
 	if (*right == 0)
 	{
 		tintin_printf2(ses, "#MAP: roomcolor is set to:  %s.", ses->map->room_list[ses->map->in_room]->color);
@@ -396,12 +382,9 @@ DO_MAP(map_get)
 DO_MAP(map_goto)
 {
 	int room;
-	char *left;
 
 	CHECK_MAP();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	room = find_room(ses, left);
 
 	if (room != -1)
@@ -461,12 +444,9 @@ DO_MAP(map_insert)
 	char temp[BUFFER_SIZE];
 	struct exit_data *exit;
 	struct listnode *node;
-	char *left;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	for (room = 1 ; room < MAX_ROOM ; room++)
 	{
 		if (ses->map->room_list[room] == NULL)
@@ -492,6 +472,12 @@ DO_MAP(map_insert)
 		return;
 	}
 
+	if (node == NULL)
+	{
+		tintin_printf2(ses, "#MAP: Given direction must be a pathdir.");
+		return;
+	}
+
 	in_room = ses->map->in_room;
 	to_room = exit->vnum;
 
@@ -524,12 +510,8 @@ DO_MAP(map_leave)
 
 DO_MAP(map_legenda)
 {
-	char *left;
-
 	CHECK_MAP();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if (*left == 0)
 	{
 		tintin_printf2(ses, "#MAP: Current legenda: %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
@@ -548,14 +530,11 @@ DO_MAP(map_legenda)
 
 DO_MAP(map_link)
 {
-	char temp[BUFFER_SIZE], *left, *right;
+	char temp[BUFFER_SIZE];
 	struct listnode *node;
 	struct exit_data *exit;
 	int room;
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-
 	CHECK_INSIDE();
 
 	if ((room = find_room(ses, right)) == -1)
@@ -609,12 +588,8 @@ DO_MAP(map_list)
 
 DO_MAP(map_map)
 {
-	char *left;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if (!strcasecmp("vt", left))
 	{
 		show_vtmap(ses);
@@ -627,23 +602,15 @@ DO_MAP(map_map)
 
 DO_MAP(map_move)
 {
-	char *left;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	follow_map(ses, left);
 }
 
 DO_MAP(map_name)
 {
-	char *left;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	free(ses->map->room_list[ses->map->in_room]->name);
 	ses->map->room_list[ses->map->in_room]->name = strdup(left);
 }
@@ -651,11 +618,9 @@ DO_MAP(map_name)
 DO_MAP(map_read)
 {
 	FILE *myfile;
-	char temp[BUFFER_SIZE], *left, *cptr;
+	char buffer[BUFFER_SIZE], *cptr;
 	int room = 0;
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if ((myfile = fopen(left, "r")) == NULL)
 	{
 		tintin_printf(ses, "#MAP: Map file {%s} not found.", left);
@@ -663,50 +628,53 @@ DO_MAP(map_read)
 		return;
 	}
 
-	SET_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+	gtd->quiet++;
 
 	if (ses->map == NULL)
 	{
 		ses->map = calloc(1, sizeof(struct map_data));
 	}
 
-	while (fgets(temp, BUFFER_SIZE - 1, myfile))
+	while (fgets(buffer, BUFFER_SIZE - 1, myfile))
 	{
-		for (cptr = temp ; *cptr && *cptr != '\n' ; cptr++);
+		cptr = strchr(buffer, '\n');
 
-		*cptr = 0;
+		if (cptr)
+		{
+			*cptr = 0;
+		}
 
-		switch (temp[0])
+		switch (buffer[0])
 		{
 			case 'C':
 				create_map(ses);
 				break;
 
 			case 'E':
-				create_exit(ses, room, &temp[2]);
+				create_exit(ses, room, &buffer[2]);
 				break;
 
 			case 'F':
-				ses->map->flags = atoi(&temp[2]);
+				ses->map->flags = atoi(&buffer[2]);
 				break;
 
 			case 'L':
-				create_legenda(ses, &temp[2]);
+				create_legenda(ses, &buffer[2]);
 				break;
 
 			case 'R':
-				room = create_room(ses, &temp[2]);
+				room = create_room(ses, &buffer[2]);
 				break;
 
 			case '#':
-				ses = parse_input(ses, temp);
+				ses = pre_parse_input(ses, buffer, SUB_NONE);
 				break;
 
 			case 0:
 				break;
 
 			default:
-				DEL_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+				gtd->quiet--;
 
 				tintin_printf2(ses, "#MAP: Invalid file, aborting.");
 
@@ -718,7 +686,7 @@ DO_MAP(map_read)
 		}
 	}
 
-	DEL_BIT(gtd->flags, TINTIN_FLAG_QUIET);
+	gtd->quiet--;
 
 	fclose(myfile);
 
@@ -727,12 +695,8 @@ DO_MAP(map_read)
 
 DO_MAP(map_roomflag)
 {
-	char *left;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if (*left)
 	{
 		if (is_abbrev(left, "avoid"))
@@ -771,13 +735,8 @@ DO_MAP(map_roomflag)
 
 DO_MAP(map_set)
 {
-	char *left, *right;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, FALSE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-
 	if (*right == 0)
 	{
 		tintin_printf2(ses, "#MAP: roomcolor is set to:  %s.", ses->map->room_list[ses->map->in_room]->color);
@@ -818,6 +777,13 @@ DO_MAP(map_set)
 	}
 }
 
+DO_MAP(map_travel)
+{
+	CHECK_INSIDE();
+
+	explore_path(ses, TRUE, left, right);
+}
+
 DO_MAP(map_undo)
 {
 	struct listnode *node;
@@ -872,12 +838,9 @@ DO_MAP(map_undo)
 DO_MAP(map_unlink)
 {
 	struct exit_data *exit;
-	char *left;
 
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	exit = find_exit(ses, ses->map->in_room, left);
 
 	if (exit == NULL)
@@ -894,27 +857,20 @@ DO_MAP(map_unlink)
 
 DO_MAP(map_walk)
 {
-	char *left, *right;
-
 	CHECK_INSIDE();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
-
 	shortest_path(ses, TRUE, left, right);
 }
 
 DO_MAP(map_write)
 {
 	FILE *file;
-	char *left, temp[BUFFER_SIZE];
+	char temp[BUFFER_SIZE];
 	struct exit_data *exit;
 	int vnum;
 
 	CHECK_MAP();
 
-	arg = get_arg_in_braces(arg, &left, TRUE);
-
 	if (*left == 0 || (file = fopen(left, "w")) == NULL)
 	{
 		tintin_printf(ses, "#ERROR: #WRITEMAP {%s} - COULDN'T OPEN FILE TO WRITE.", left);
@@ -1005,11 +961,11 @@ void delete_map(struct session *ses)
 void create_legenda(struct session *ses, char *arg)
 {
 	int cnt;
-	char *buf;
+	char buf[BUFFER_SIZE];
 
 	for (cnt = 0 ; cnt < 17 ; cnt++)
 	{
-		arg = get_arg_in_braces(arg, &buf, FALSE);
+		arg = get_arg_in_braces(arg, buf, FALSE);
 
 		if (*buf)
 		{
@@ -1021,14 +977,14 @@ void create_legenda(struct session *ses, char *arg)
 
 int create_room(struct session *ses, char *arg)
 {
-	char *vnum, *flags, *color, *name, *symbol;
+	char vnum[BUFFER_SIZE], flags[BUFFER_SIZE], color[BUFFER_SIZE], name[BUFFER_SIZE], symbol[BUFFER_SIZE];
 	struct room_data *newroom;
 
-	arg = get_arg_in_braces(arg, &vnum,   FALSE);
-	arg = get_arg_in_braces(arg, &flags,  FALSE);
-	arg = get_arg_in_braces(arg, &color,  FALSE);
-	arg = get_arg_in_braces(arg, &name,   FALSE);
-	arg = get_arg_in_braces(arg, &symbol, FALSE);
+	arg = get_arg_in_braces(arg, vnum,   FALSE);
+	arg = get_arg_in_braces(arg, flags,  FALSE);
+	arg = get_arg_in_braces(arg, color,  FALSE);
+	arg = get_arg_in_braces(arg, name,   FALSE);
+	arg = get_arg_in_braces(arg, symbol, FALSE);
 
 	/* Backward compatbility 1.96.5 */
 
@@ -1090,11 +1046,11 @@ void delete_room(struct session *ses, int room, int exits)
 void create_exit(struct session *ses, int room, char *arg)
 {
 	struct exit_data *newexit;
-	char *vnum, *name, *cmd;
+	char vnum[BUFFER_SIZE], name[BUFFER_SIZE], cmd[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, &vnum, FALSE);
-	arg = get_arg_in_braces(arg, &name, FALSE);
-	arg = get_arg_in_braces(arg, &cmd,  TRUE);
+	arg = get_arg_in_braces(arg, vnum, FALSE);
+	arg = get_arg_in_braces(arg, name, FALSE);
+	arg = get_arg_in_braces(arg, cmd,  TRUE);
 
 	newexit       = calloc(1, sizeof(struct exit_data));
 	newexit->vnum = atoi(vnum);
@@ -1164,10 +1120,9 @@ void follow_map(struct session *ses, char *argument)
 		{
 			show_message(ses, LIST_MAP, "#MAP: LEAVE FLAG FOUND IN ROOM {%d}. LEAVING MAP.", ses->map->in_room);
 
-			map_leave(ses, argument);
+			map_leave(ses, "", "");
 		}
-
-		if (HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_VOID))
+		else if (HAS_BIT(ses->map->room_list[ses->map->in_room]->flags, ROOM_FLAG_VOID))
 		{
 			node = searchnode_list(ses->list[LIST_PATHDIR], argument);
 
@@ -1246,16 +1201,16 @@ void follow_map(struct session *ses, char *argument)
 
 void insert_undo(struct session *ses, char *format, ...)
 {
-	char buf[BUFFER_SIZE], *arg, *dir, *rev, *flag;
+	char buf[BUFFER_SIZE], *arg, dir[BUFFER_SIZE], rev[BUFFER_SIZE], flag[BUFFER_SIZE];
 	va_list args;
 
 	va_start(args, format);
 	vsprintf(buf, format, args);
 	va_end(args);
 
-	arg = get_arg_in_braces(buf, &dir, FALSE);
-	arg = get_arg_in_braces(arg, &rev, FALSE);
-	arg = get_arg_in_braces(arg, &flag, FALSE);
+	arg = get_arg_in_braces(buf, dir, FALSE);
+	arg = get_arg_in_braces(arg, rev, FALSE);
+	arg = get_arg_in_braces(arg, flag, FALSE);
 
 	addnode_list(ses->list[LIST_MAP], dir, rev, flag);
 
@@ -1340,7 +1295,7 @@ void create_map_grid(struct session *ses, short room, short x, short y)
 
 void show_map(struct session *ses, char *argument)
 {
-	char buf[BUFFER_SIZE], *out;
+	char buf[BUFFER_SIZE], out[BUFFER_SIZE];
 	int x, y, line, size;
 
 	push_call("show_map(%p,%p)",ses,argument);
@@ -1373,7 +1328,7 @@ void show_map(struct session *ses, char *argument)
 					strcat(buf, draw_room(ses, map_grid[x][y], line));
 				}
 
-				substitute(ses, buf, &out, SUB_COL);
+				substitute(ses, buf, out, SUB_COL);
 
 				tintin_puts2(ses, out);
 			}
@@ -1391,7 +1346,7 @@ void show_map(struct session *ses, char *argument)
 			strcat(buf, draw_room(ses, map_grid[x][y], 0));
 		}
 
-		substitute(ses, buf, &out, SUB_COL);
+		substitute(ses, buf, out, SUB_COL);
 
 		tintin_puts2(ses, out);
 	}
@@ -1403,7 +1358,7 @@ void show_map(struct session *ses, char *argument)
 
 void show_vtmap(struct session *ses)
 {
-	char buf[BUFFER_SIZE], *out;
+	char buf[BUFFER_SIZE], out[BUFFER_SIZE];
 	int x, y, line;
 
 	push_call("show_vtmap(%p)",ses);
@@ -1448,7 +1403,7 @@ void show_vtmap(struct session *ses)
 				{
 					strcat(buf, draw_room(ses, map_grid[x][y], line));
 				}
-				substitute(ses, buf, &out, SUB_COL);
+				substitute(ses, buf, out, SUB_COL);
 
 				printf("%s\033[0K\r\n", out);
 			}
@@ -1464,7 +1419,7 @@ void show_vtmap(struct session *ses)
 			{
 				strcat(buf, draw_room(ses, map_grid[x][y], 0));
 			}
-			substitute(ses, buf, &out, SUB_COL);
+			substitute(ses, buf, out, SUB_COL);
 
 			printf("%s\r\n", out);
 		}
@@ -1743,7 +1698,7 @@ void search_path(short room, short size)
 
 void shortest_path(struct session *ses, int walk, char *left, char *right)
 {
-	char time[BUFFER_SIZE], delay[BUFFER_SIZE];
+	char time[BUFFER_SIZE], name[BUFFER_SIZE];
 	struct exit_data *exit;
 	int room, vnum, wait;
 
@@ -1843,13 +1798,12 @@ void shortest_path(struct session *ses, int walk, char *left, char *right)
 
 			while (ses->list[LIST_PATH]->f_node)
 			{
-				sprintf(time, "%lld", utime());
+				sprintf(time, "%lld", utime() + wait);
+				sprintf(name, "%lld", gtd->time);
 
-				wait += atoi(right);
+				wait += (long long) (get_number(ses, left) * 1000000LL);
 
-				sprintf(delay, "%d", wait);
-
-				updatenode_list(ses, time, ses->list[LIST_PATH]->f_node->left, delay, LIST_DELAY);
+				updatenode_list(ses, name, ses->list[LIST_PATH]->f_node->left, time, LIST_DELAY);
 
 				deletenode_list(ses, ses->list[LIST_PATH]->f_node, LIST_PATH);
 			}
@@ -1858,7 +1812,7 @@ void shortest_path(struct session *ses, int walk, char *left, char *right)
 		{
 			while (ses->list[LIST_PATH]->f_node)
 			{
-				parse_input(ses, ses->list[LIST_PATH]->f_node->left);
+				pre_parse_input(ses, ses->list[LIST_PATH]->f_node->left, SUB_NONE);
 
 				deletenode_list(ses, ses->list[LIST_PATH]->f_node, LIST_PATH);
 			}
@@ -1945,3 +1899,79 @@ void search_coord(int vnum, short x, short y, short z)
 		}
 	}
 }
+
+void explore_path(struct session *ses, int walk, char *left, char *right)
+{
+	char time[BUFFER_SIZE], name[BUFFER_SIZE];
+	struct exit_data *exit;
+	int room, vnum, wait;
+
+	if (HAS_BIT(ses->flags, SES_FLAG_MAPPING))
+	{
+		tintin_printf2(ses, "#MAP EXPLORE: You have to stop #PATH TRACKING first.");
+
+		return;
+	}
+
+	kill_list(ses, LIST_PATH);
+
+	room = ses->map->in_room;
+
+	exit = find_exit(ses, room, left);
+
+	if (exit == NULL)
+	{
+		tintin_printf2(ses, "#MAP: There's no exit named '%s'.", left);
+		return;
+	}
+
+	vnum = exit->vnum;
+
+	addnode_list(ses->list[LIST_PATH], exit->cmd, "", "0");
+
+	while (get_map_exits(ses, vnum) == 2)
+	{
+		if (ses->map->room_list[vnum]->f_exit->vnum != room)
+		{
+			exit = ses->map->room_list[vnum]->f_exit;
+		}
+		else
+		{
+			exit = ses->map->room_list[vnum]->l_exit;
+		}
+
+		addnode_list(ses->list[LIST_PATH], exit->cmd, "", "0");
+
+		room = vnum;
+		vnum = exit->vnum;
+	}
+
+	if (walk)
+	{
+		if (*right)
+		{
+			wait = 0;
+
+			while (ses->list[LIST_PATH]->f_node)
+			{
+				sprintf(time, "%lld", utime() + wait);
+				sprintf(name, "%lld", gtd->time);
+
+				wait += (long long) (get_number(ses, left) * 1000000LL);
+
+				updatenode_list(ses, name, ses->list[LIST_PATH]->f_node->left, time, LIST_DELAY);
+
+				deletenode_list(ses, ses->list[LIST_PATH]->f_node, LIST_PATH);
+			}
+		}
+		else
+		{
+			while (ses->list[LIST_PATH]->f_node)
+			{
+				pre_parse_input(ses, ses->list[LIST_PATH]->f_node->left, SUB_NONE);
+
+				deletenode_list(ses, ses->list[LIST_PATH]->f_node, LIST_PATH);
+			}
+		}
+	}
+}
