@@ -30,17 +30,18 @@
 
 DO_COMMAND(do_all)
 {
-	char *left;
+	char left[BUFFER_SIZE];
 	struct session *sesptr, *next_ses;
 
 	if (gts->next)
 	{
-		get_arg_in_braces(arg, &left, TRUE);
+		get_arg_in_braces(arg, left, TRUE);
 
 		for (sesptr = gts->next ; sesptr ; sesptr = next_ses)
 		{
 			next_ses = sesptr->next;
-			parse_input(sesptr, left);
+
+			pre_parse_input(sesptr, left, SUB_NONE);
 		}
 	}
 	else
@@ -92,7 +93,7 @@ DO_COMMAND(do_commands)
 
 DO_COMMAND(do_cr)
 {
-	write_line_mud("\r\n", ses);
+	write_mud(ses, "", SUB_EOL);
 
 	return ses;
 }
@@ -100,15 +101,15 @@ DO_COMMAND(do_cr)
 
 DO_COMMAND(do_echo)
 {
-	char *temp;
+	char temp[BUFFER_SIZE];
 
-	temp = stringf_alloc("result %s", arg);
+	sprintf(temp, "result %s", arg);
 
 	internal_variable(ses, "{result}");
 
 	do_format(ses, temp);
 
-	substitute(ses, "$result", &temp, SUB_VAR);
+	substitute(ses, "$result", temp, SUB_VAR);
 
 	if (*temp)
 	{
@@ -135,12 +136,12 @@ DO_COMMAND(do_end)
 
 DO_COMMAND(do_forall)
 {
-	char *left, *right, *temp;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
 
-	arg = get_arg_in_braces(arg, &temp,  TRUE);
-	arg = get_arg_in_braces(arg, &right, TRUE);
+	arg = get_arg_in_braces(arg, left,  TRUE);
+	arg = get_arg_in_braces(arg, right, TRUE);
 
-	substitute(ses, temp, &left, SUB_VAR|SUB_FUN);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
 	if (*left == 0 || *right == 0)
 	{
@@ -152,13 +153,13 @@ DO_COMMAND(do_forall)
 
 		while (*arg)
 		{
-			arg = get_arg_in_braces(arg, &gtd->cmds[0], FALSE);
+			arg = get_arg_in_braces(arg, temp, FALSE);
 
-			internal_variable(ses, "{forall} {%s}", gtd->cmds[0]);
+			internal_variable(ses, "{forall} {%s}", temp);
 
-			substitute(ses, right, &temp, SUB_CMD);
+			RESTRING(gtd->cmds[0], temp);
 
-			ses = parse_input(ses, temp);
+			ses = pre_parse_input(ses, right, SUB_CMD);
 		}
 	}
 	return ses;
@@ -177,6 +178,8 @@ DO_COMMAND(do_info)
 {
 	int cnt;
 
+	printf("--%s--\n", arg);
+
 	if (*arg == 'c')
 	{
 		show_cpu(ses);
@@ -207,15 +210,15 @@ DO_COMMAND(do_info)
 
 DO_COMMAND(do_loop)
 {
-	char *left, *right, *temp, *command;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE], command[BUFFER_SIZE];
 
 	long long bound1, bound2, counter, step;
 
-	arg = get_arg_in_braces(arg, &temp, FALSE);
-	arg = get_arg_in_braces(arg, &command, TRUE);
+	arg = get_arg_in_braces(arg, temp,    0);
+	arg = get_arg_in_braces(arg, command, 1);
 
-	arg = get_arg_in_braces(temp, &left, FALSE);
-	arg = get_arg_in_braces(arg, &right, FALSE);
+	arg = get_arg_in_braces(temp, left, 0);
+	arg = get_arg_in_braces(arg, right, 0);
 
 
 	if (*command == 0 || (*left == 0 && *right == 0))
@@ -243,19 +246,18 @@ DO_COMMAND(do_loop)
 
 		for (counter = bound1 ; counter != bound2 ; counter += step)
 		{
-			gtd->cmds[0] = stringf_alloc("%lld", counter);
+			gtd->cmds[0] = refstring(gtd->cmds[0], "%lld", counter);
 
 			internal_variable(ses, "{loop} {%lld}", counter);
 
-			substitute(ses, command, &temp, SUB_CMD);
-			ses = parse_input(ses, temp);
+			ses = pre_parse_input(ses, command, SUB_CMD);
 		}
 	}
 	else
 	{
 		while ((long long) get_number(ses, left))
 		{
-			ses = parse_input(ses, command);
+			ses = pre_parse_input(ses, command, SUB_NONE);
 		}
 	}
 	return ses;
@@ -270,13 +272,13 @@ DO_COMMAND(do_nop)
 
 DO_COMMAND(do_parse)
 {
-	char *left, *right, *temp;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
 	int cnt;
 
-	arg = get_arg_in_braces(arg, &temp, 0);
-	substitute(ses, temp, &left, SUB_VAR|SUB_FUN);
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 1);
 
-	arg = get_arg_in_braces(arg, &right, 1);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
 	if (*left == 0 || *right == 0)
 	{
@@ -289,20 +291,21 @@ DO_COMMAND(do_parse)
 #ifdef BIG5
 			if (left[cnt] & 0x80 && left[cnt+1] != 0)
 			{
-				gtd->cmds[0] = stringf_alloc("%c%c", left[cnt], left[++cnt]);
+				sprintf(temp, "%c%c", left[cnt], left[++cnt]));
 			}
 			else
 			{
-				gtd->cmds[0] = stringf_alloc("%c", left[cnt]);
+				sprintf(temp, "%c", left[cnt]);
 			}
 #else
-			gtd->cmds[0] = stringf_alloc("%c", left[cnt]);
+			sprintf(temp, "%c", left[cnt]);
 #endif
+
+			RESTRING(gtd->cmds[0], temp);
+
 			internal_variable(ses, "{parse} {%s}", gtd->cmds[0]);
 
-			substitute(ses, right, &temp, SUB_CMD);
-
-			ses = parse_input(ses, temp);
+			ses = pre_parse_input(ses, right, SUB_CMD);
 		}
 	}
 	return ses;
@@ -315,33 +318,30 @@ DO_COMMAND(do_return)
 
 	if (*arg)
 	{
-		internal_variable(ses, "{result} {%s}", arg);
+		internal_variable(ses, "{result} %s", arg);
 	}
 	return ses;
 }
 
-
 DO_COMMAND(do_send)
 {
-	char *left, *out;
+	char left[BUFFER_SIZE];
 
-	get_arg_in_braces(arg, &left, TRUE);
+	get_arg_in_braces(arg, left, TRUE);
 
-	substitute(ses, left, &out, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
-
-	write_line_mud(out, ses);
+	write_mud(ses, left, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 
 	return ses;
 }
 
 DO_COMMAND(do_showme)
 {
-	char *left, *right, *temp, *output;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], output[STRING_SIZE];
 
-	arg = get_arg_in_braces(arg, &temp, TRUE);
-	arg = get_arg_in_braces(arg, &right, FALSE);
+	arg = get_arg_in_braces(arg, left,  1);
+	arg = get_arg_in_braces(arg, right, 0);
 
-	substitute(ses, temp, &left, SUB_COL|SUB_ESC);
+	substitute(ses, left, left, SUB_COL|SUB_ESC);
 
 	if (*right)
 	{
@@ -349,9 +349,7 @@ DO_COMMAND(do_showme)
 
 		return ses;
 	}
-	do_one_line(&left, ses);
-
-	substitute(ses, left, &temp, SUB_COL|SUB_ESC);
+	do_one_line(left, ses);
 
 	if (HAS_BIT(ses->flags, SES_FLAG_GAG))
 	{
@@ -362,11 +360,11 @@ DO_COMMAND(do_showme)
 
 	if (strip_vt102_strlen(ses->more_output) != 0)
 	{
-		output = stringf_alloc("\r\n\033[0m%s\033[0m", temp);
+		sprintf(output, "\r\n\033[0m%s\033[0m", left);
 	}
 	else
 	{
-		output = stringf_alloc("\033[0m%s\033[0m", temp);
+		sprintf(output, "\033[0m%s\033[0m", left);
 	}
 
 	add_line_buffer(ses, output, FALSE);
@@ -395,9 +393,9 @@ DO_COMMAND(do_showme)
 DO_COMMAND(do_snoop)
 {
 	struct session *sesptr = ses;
-	char *left;
+	char left[BUFFER_SIZE];
 
-	get_arg_in_braces(arg, &left, 1);
+	get_arg_in_braces(arg, left, 1);
 
 	if (*left)
 	{
@@ -440,37 +438,6 @@ DO_COMMAND(do_suspend)
 	return ses;
 }
 
-DO_COMMAND(do_system)
-{
-	char *left;
-
-	get_arg_in_braces(arg, &left, TRUE);
-
-	if (!*left)
-	{
-		tintin_printf(ses, "#SYNTAX: #SYSTEM {COMMAND}");
-		return ses;
-	}
-
-	show_message(ses, -1, "#OK: EXECUTING '%s'", left);
-
-	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
-	{
-		save_pos(ses);
-		goto_rowcol(ses, ses->bot_row, 1);
-	}
-	fflush(stdout);
-
-	system(left);
-
-	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
-	{
-		restore_pos(ses);
-	}
-
-	return ses;
-}
-
 
 DO_COMMAND(do_zap)
 {
