@@ -164,15 +164,21 @@ int connect_mud(char *host, char *port, struct session *ses)
 
 void write_line_mud(char *line, struct session *ses)
 {
+	push_call("write_line_mud(%p,%p)",line,ses);
+
 	if (ses == gts)
 	{
 		tintin_printf2(ses, "#NO SESSION ACTIVE. USE: %csession {name} {host} {port} TO START ONE.", gtd->tintin_char);
+
+		pop_call();
 		return;
 	}
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_CONNECTED))
 	{
 		tintin_printf2(ses, "#THIS SESSION IS NOT CONNECTED.");
+
+		pop_call();
 		return;
 	}
 
@@ -180,65 +186,46 @@ void write_line_mud(char *line, struct session *ses)
 	{
 		syserr("write in write_line_mud");
 	}
+	pop_call();
 	return;
 }
 
 
-void read_buffer_mud(struct session *ses)
+int read_buffer_mud(struct session *ses)
 {
 	unsigned char buf[STRING_SIZE];
 	int size;
 
-	gtd->mud_output_len = 0;
+	push_call("read_buffer_mud(%p)",ses);
 
 	size = read(ses->socket, buf, STRING_SIZE - 1);
 
 	if (size <= 0)
 	{
-		gtd->mud_output_len = -1;
-		return;
+		pop_call();
+		return FALSE;
 	}
 
 	buf[size] = 0;
 
 	translate_telopts(ses, buf, size);
 
-	gtd->mud_output_buf[gtd->mud_output_len] = 0;
-
-	return;
+	pop_call();
+	return TRUE;
 }
 
 
 void readmud(struct session *ses)
 {
-	char *line, *next_line, *unwind;
+	char *line, *next_line;
 	char linebuf[STRING_SIZE];
 
 	push_call("readmud(%p)", ses);
 
-	if (!HAS_BIT(ses->flags, SES_FLAG_SCAN))
-	{
-		read_buffer_mud(ses);
-
-		switch (gtd->mud_output_len)
-		{
-			case -1:
-				cleanup_session(ses);
-
-				pop_call();
-				return;
-
-			case  0:
-				pop_call();
-				return;
-		}
-	}
 	gtd->mud_output_len = 0;
 
 	/* separate into lines and print away */
 
-	unwind = string_alloc("");
-
 	if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
 	{
 		save_pos(gtd->ses);
@@ -261,7 +248,6 @@ void readmud(struct session *ses)
 			break;
 		}
 
-
 		if (next_line == NULL && strlen(ses->more_output) < BUFFER_SIZE / 2)
 		{
 			if (!HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_PROMPT))
@@ -275,15 +261,18 @@ void readmud(struct session *ses)
 			}
 		}
 
-
 		if (*line && ses->more_output[0])
 		{
-			if (ses->check_output || HAS_BIT(ses->flags, SES_FLAG_SPLIT))
+			if (ses->check_output)
 			{
 				sprintf(linebuf, "%s%s", ses->more_output, line);
 
 				ses->more_output[0] = 0;
 			}
+			else if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
+			{
+				sprintf(linebuf, "%s%s", ses->more_output, line);
+			}
 			else
 			{
 				strcpy(linebuf, line);
@@ -296,8 +285,6 @@ void readmud(struct session *ses)
 
 		process_mud_output(ses, linebuf, next_line == NULL);
 	}
-	DEL_BIT(gtd->ses->telopts, TELOPT_FLAG_PROMPT);
-
 	DEL_BIT(gtd->ses->flags, SES_FLAG_READMUD);
 
 	if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
@@ -305,8 +292,6 @@ void readmud(struct session *ses)
 		restore_pos(gtd->ses);
 	}
 
-	string_free(unwind);
-
 	pop_call();
 	return;
 }
@@ -314,9 +299,20 @@ void readmud(struct session *ses)
 
 void process_mud_output(struct session *ses, char *linebuf, int prompt)
 {
+	char line[STRING_SIZE];
+
 	ses->check_output = 0;
 
-	do_one_line(&linebuf, ses);   /* changes linebuf */
+	if (HAS_BIT(ses->flags, SES_FLAG_COLORPATCH))
+	{
+		get_color_codes(ses->color, linebuf, ses->color);
+
+		sprintf(line, "%s%s", ses->color, linebuf);
+
+		linebuf = line;
+	}
+
+	do_one_line(linebuf, ses);   /* changes linebuf */
 
 	/*
 		Take care of gags, vt102 support still goes
