@@ -70,13 +70,16 @@ struct command_type command_table[] =
 	{	"message",           do_message,           CMD_FLAG_SUB     },
 	{	"nop",               do_nop,               CMD_FLAG_NONE    },
 	{	"parse",             do_parse,             CMD_FLAG_NONE    },
-	{	"path",              do_path,              CMD_FLAG_NONE    },
+	{	"path",              do_path,              CMD_FLAG_SUB     },
 	{	"pathdir",           do_pathdir,           CMD_FLAG_NONE    },
 	{	"prompt",            do_prompt,            CMD_FLAG_NONE    },
 	{	"read",              do_read,              CMD_FLAG_SUB     },
+	{    "regexp",            do_regexp,            CMD_FLAG_NONE    },
 	{	"replacestring",     do_replacestring,     CMD_FLAG_SUB     },
 	{	"return",            do_return,            CMD_FLAG_NONE    },
+	{    "run",               do_run,               CMD_FLAG_SUB     },
 	{	"scan",              do_scan,              CMD_FLAG_SUB     },
+	{    "script",            do_script,            CMD_FLAG_SUB     },
 	{	"send",              do_send,              CMD_FLAG_NONE    },
 	{	"session",           do_session,           CMD_FLAG_SUB     },
 	{	"showme",            do_showme,            CMD_FLAG_SUB     },
@@ -263,6 +266,13 @@ struct config_type config_table[] =
 		config_loglevel
 	},
 
+	{
+		"COLOR PATCH",
+		"Tintin will properly color the start of each line.",
+		"Tintin will leave color handling to the server.",
+		config_colorpatch
+	},
+
 	{
 		"",
 		"",
@@ -281,6 +291,7 @@ struct color_type color_table[] =
 	{    "blink",          "5", "<588>" },
 	{    "reverse",        "7", "<788>" },
 	{    "dim",           "22", "<288>" },
+	{    "dark",          "22", "<288>" },
 	{    "black",         "30", "<808>" },
 	{    "red",           "31", "<818>" },
 	{    "green",         "32", "<828>" },
@@ -312,83 +323,85 @@ struct class_type class_table[] =
 
 struct chat_type chat_table[] =
 {
-	{     "ACCEPT",          chat_accept,         "Accept a file transfer"		               },
-	{     "CALL",            chat_call,           "Call a buddy"                                   },
-	{     "CANCELFILE",      chat_cancelfile,     "Cancel a file transfer"                         },
-	{     "COLOR",           chat_color,          "Set the default chat color"                     },
-	{     "DECLINE",         chat_decline,        "Decline a file transfer"                        },
-	{     "DND",             chat_dnd,            "Decline new connections"                        },
-	{     "DOWNLOADDIR",     chat_downloaddir,    "Set the download directory"                     },
-	{     "EMOTE",           chat_emote,          "Send an emoted chat message"                    },
-	{     "FORWARD",         chat_forward,        "Forward all chat messages to a buddy"           },
-	{     "FORWARDALL",      chat_forwardall,     "Forward all chat/mud messages to a buddy"       },
-	{     "FILESTAT",        chat_filestat,       "Show file transfer data"                        },
-	{     "GROUP",           chat_group,          "Assign a group to a buddy"                      },
-	{     "IGNORE",          chat_ignore,         "Ignore all messages from a buddy"               },
-	{     "INITIALIZE",      chat_initialize,     "Initialize chat with an optional port number"   },
-	{     "INFO",            chat_info,           "Display your chat settings"                     },
-	{     "IP",              chat_ip,             "Change your IP address, unset by default"       },
-	{     "MESSAGE",         chat_message,        "Send a private message to a buddy"              },
-	{     "NAME",            chat_name,           "Change your chat name"                          },
-	{     "PASTE",           chat_paste,          "Paste a block of text to a buddy"               },
-	{     "PEEK",            chat_peek,           "Show a buddy's public connections"              },
-	{     "PING",            chat_ping,           "Display a buddy's response time"                },
-	{     "PRIVATE",         chat_private,        "Do not share a buddy's IP address"              },
-	{     "PUBLIC",          chat_public,         "Share a buddy's IP address"                     },
-	{     "REPLY",           chat_reply,          "Reply to last private message"                  },
-	{     "REQUEST",         chat_request,        "Request a buddy's public connections"           },
-	{     "SEND",            chat_send,           "Send a raw data message to a buddy"             },
-	{     "SENDFILE",        chat_sendfile,       "Send a file to a buddy"                         },
-	{     "SERVE",           chat_serve,          "Forward all public chat messages to a buddy"    },
-	{     "UNINITIALIZE",    chat_uninitialize,   "Uninitializes the chat server"                  },
-	{     "WHO",             chat_who,            "Show all connections"                           },
-	{     "ZAP",             chat_zap,            "Close the connection to a buddy"                },
-	{     "",                NULL,                ""                                               }
+	{     "ACCEPT",          chat_accept,         1, 0, "Accept a file transfer"		               },
+	{     "CALL",            chat_call,           0, 0, "Call a buddy"                                   },
+	{     "CANCELFILE",      chat_cancelfile,     1, 0, "Cancel a file transfer"                         },
+	{     "COLOR",           chat_color,          1, 0, "Set the default chat color"                     },
+	{     "DECLINE",         chat_decline,        1, 0, "Decline a file transfer"                        },
+	{     "DND",             chat_dnd,            0, 0, "Decline new connections"                        },
+	{     "DOWNLOADDIR",     chat_downloaddir,    1, 0, "Set the download directory"                     },
+	{     "EMOTE",           chat_emote,          0, 1, "Send an emoted chat message"                    },
+	{     "FORWARD",         chat_forward,        1, 0, "Forward all chat messages to a buddy"           },
+	{     "FORWARDALL",      chat_forwardall,     1, 0, "Forward all chat/mud messages to a buddy"       },
+	{     "FILESTAT",        chat_filestat,       1, 0, "Show file transfer data"                        },
+	{     "GROUP",           chat_group,          0, 1, "Assign a group to a buddy"                      },
+	{     "IGNORE",          chat_ignore,         1, 0, "Ignore all messages from a buddy"               },
+	{     "INITIALIZE",      chat_initialize,     1, 0, "Initialize chat with an optional port number"   },
+	{     "INFO",            chat_info,           0, 0, "Display your chat settings"                     },
+	{     "IP",              chat_ip,             1, 0, "Change your IP address, unset by default"       },
+	{     "MESSAGE",         chat_message,        0, 1, "Send a private message to a buddy"              },
+	{     "NAME",            chat_name,           1, 0, "Change your chat name"                          },
+	{     "PASTE",           chat_paste,          0, 1, "Paste a block of text to a buddy"               },
+	{     "PEEK",            chat_peek,           1, 0, "Show a buddy's public connections"              },
+	{     "PING",            chat_ping,           1, 0, "Display a buddy's response time"                },
+	{     "PRIVATE",         chat_private,        1, 0, "Do not share a buddy's IP address"              },
+	{     "PUBLIC",          chat_public,         1, 0, "Share a buddy's IP address"                     },
+	{     "REPLY",           chat_reply,          1, 0, "Reply to last private message"                  },
+	{     "REQUEST",         chat_request,        1, 0, "Request a buddy's public connections"           },
+	{     "SEND",            chat_send,           0, 1, "Send a raw data message to a buddy"             },
+	{     "SENDFILE",        chat_sendfile,       0, 1, "Send a file to a buddy"                         },
+	{     "SERVE",           chat_serve,          1, 0, "Forward all public chat messages to a buddy"    },
+	{     "UNINITIALIZE",    chat_uninitialize,   0, 0, "Uninitializes the chat server"                  },
+	{     "WHO",             chat_who,            0, 0, "Show all connections"                           },
+	{     "ZAP",             chat_zap,            1, 0, "Close the connection to a buddy"                },
+	{     "",                NULL,                0, 0, ""                                               }
 };
 
 
 struct array_type array_table[] =
 {
-	{     "CLR",                       array_clr              },
-	{     "DEL",                       array_del              },
-	{     "FND",                       array_fnd              },
-	{     "GET",                       array_get              },
-	{     "INS",                       array_ins              },
-	{     "LEN",                       array_len              },
-	{     "SET",                       array_set              },
-	{     "SRT",                       array_srt              },
-	{     "",                          NULL                   }
+	{     "CLR",             array_clr,           0, 0    },
+	{     "DEL",             array_del,           0, 0    },
+	{     "FND",             array_fnd,           0, 1    },
+	{     "GET",             array_get,           0, 1    },
+	{     "INS",             array_ins,           0, 1    },
+	{     "LEN",             array_len,           1, 0    },
+	{     "SET",             array_set,           0, 1    },
+	{     "SRT",             array_srt,           1, 0    },
+	{     "",                NULL,                0, 0    }
 };
 
 struct map_type map_table[] =
 {
-	{     "COLOR",                     map_color              },
-	{     "CREATE",                    map_create             },
-	{     "DELETE",                    map_delete             },
-	{     "DESTROY",                   map_destroy            },
-	{     "DIG",                       map_dig                },
-	{     "EXIT",                      map_exit               },
-	{     "FIND",                      map_find               },
-	{     "FLAG",                      map_flag               },
-	{     "GET",                       map_get                },
-	{     "GOTO",                      map_goto               },
-	{     "INFO",                      map_info               },
-	{     "INSERT",                    map_insert             },
-	{     "LEAVE",                     map_leave              },
-	{     "LEGENDA",                   map_legenda            },
-	{     "LINK",                      map_link               },
-	{     "LIST",                      map_list               },
-	{     "MAP",                       map_map                },
-	{     "MOVE",                      map_move               },
-	{     "NAME",                      map_name               },
-	{     "READ",                      map_read               },
-	{     "ROOMFLAG",                  map_roomflag           },
-	{     "SET",                       map_set                },
-	{     "UNDO",                      map_undo               },
-	{     "UNLINK",                    map_unlink             },
-	{     "WALK",                      map_walk               },
-	{     "WRITE",                     map_write              },
-	{     "",                          NULL                   }
+	{     "COLOR",                     map_color,              1,1},
+	{     "CREATE",                    map_create,             1,1},
+	{     "DELETE",                    map_delete,             1,1},
+	{     "DESTROY",                   map_destroy,            1,1},
+	{     "DIG",                       map_dig,                1,1},
+	{     "EXIT",                      map_exit,               0,1},
+	{     "EXPLORE",                   map_explore,            1,1},
+	{     "FIND",                      map_find,               1,1},
+	{     "FLAG",                      map_flag,               1,1},
+	{     "GET",                       map_get,                1,1},
+	{     "GOTO",                      map_goto,               1,1},
+	{     "INFO",                      map_info,               1,1},
+	{     "INSERT",                    map_insert,             1,1},
+	{     "LEAVE",                     map_leave,              1,1},
+	{     "LEGENDA",                   map_legenda,            1,1},
+	{     "LINK",                      map_link,               0,1},
+	{     "LIST",                      map_list,               1,1},
+	{     "MAP",                       map_map,                1,1},
+	{     "MOVE",                      map_move,               1,1},
+	{     "NAME",                      map_name,               1,1},
+	{     "READ",                      map_read,               1,1},
+	{     "ROOMFLAG",                  map_roomflag,           1,1},
+	{     "SET",                       map_set,                0,1},
+	{     "TRAVEL",                    map_travel,             1,1},
+	{     "UNDO",                      map_undo,               1,1},
+	{     "UNLINK",                    map_unlink,             1,1},
+	{     "WALK",                      map_walk,               1,1},
+	{     "WRITE",                     map_write,              1,1},
+	{     "",                          NULL,                   1,1}
 };
 
 struct cursor_type cursor_table[] =
@@ -666,6 +679,7 @@ struct path_type path_table[] =
 	{    "LOAD",                       path_load              },
 	{    "MAP",                        path_map               },
 	{    "NEW",                        path_new               },
+	{    "RUN",                        path_run               },
 	{    "SAVE",                       path_save              },
 	{    "WALK",                       path_walk              },
 	{    "",                           NULL                   }
