@@ -26,6 +26,8 @@
 *                     recoded by Igor van den Hoven 2004                      *
 ******************************************************************************/
 
+#include <regex.h>
+
 #include "tintin.h"
 
 
@@ -102,18 +104,84 @@ int regexp(char *exp, char *str, unsigned char cs)
 	return (*exp == *str);
 }
 
+DO_COMMAND(do_regexp)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], true[BUFFER_SIZE], false[BUFFER_SIZE], str[BUFFER_SIZE], exp[BUFFER_SIZE];
+
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 0);
+	arg = get_arg_in_braces(arg, true,  1);
+	arg = get_arg_in_braces(arg, false, 1);
+
+	if (*left == 0 || *right == 0 || *true == 0)
+	{
+		tintin_printf2(ses, "SYNTAX: #REGEXP {string} {expression} {true} {false}");
+	}
+	else
+	{
+		substitute(ses, left,  str, SUB_VAR|SUB_FUN);
+		substitute(ses, right, exp, SUB_VAR|SUB_FUN);
+
+		if (regex(exp, str))
+		{
+			ses = pre_parse_input(ses, true, SUB_CMD);
+		}
+		else if (*false)
+		{
+			ses = pre_parse_input(ses, false, SUB_NONE);
+		}
+	}
+	return ses;
+}
+		
+int regex(char *exp, char *str)
+{
+	regex_t regex;
+	regmatch_t match[100];
+	int i;
+
+	if (regcomp(&regex, exp, REG_EXTENDED))
+	{
+		return FALSE;
+	}
+
+	if (regexec(&regex, str, 100, match, 0))
+	{
+		regfree(&regex);
+
+		return FALSE;
+	}
+
+	for (i = 0 ; i < 100 ; i++)
+	{
+		if (match[i].rm_so != -1)
+		{
+			gtd->cmds[i] = refstring(gtd->cmds[i], "%.*s", match[i].rm_eo - match[i].rm_so, &str[match[i].rm_so]);
+		}
+		else
+		{
+			break;
+		}
+	}
+	regfree(&regex);
+
+	return TRUE;
+}
+
 /******************************************************************************
 * copy *string into *result, but substitute the various expressions with the  *
 * values they stand for.                                                      *
 ******************************************************************************/
 
-void substitute(struct session *ses, char *string, char **result, int flags)
+void substitute(struct session *ses, char *string, char *result, int flags)
 {
 	struct listnode *node;
-	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[STRING_SIZE], *pti, *pto, *ptt, *tmp;
+	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[STRING_SIZE], *pti, *pto, *ptt;
 	char *pte;
 	int i, cnt, flags_neol = flags;
 
+	push_call("substitute(%p,%p,%p,%d)",ses,string,result,flags);
+
 	pti = string;
 	pto = buffer;
 
@@ -144,8 +212,9 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 				}
 				*pto = 0;
 
-				*result = string_alloc(buffer);
+				strcpy(result, buffer);
 
+				pop_call();
 				return;
 
 			case '$':
@@ -161,8 +230,7 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 						if (pti[i] == DEFAULT_CLOSE)
 						{
 							*ptt = 0;
-							substitute(ses, buf, &tmp, flags_neol);
-							strcpy(temp, tmp);
+							substitute(ses, buf, temp, flags_neol);
 						}
 						else
 						{
@@ -182,8 +250,7 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 
 					if ((node = searchnode_list(ses->list[LIST_VARIABLE], temp)) != NULL)
 					{
-						substitute(ses, node->right, &tmp, flags_neol - SUB_VAR);
-						strcpy(pto, tmp);
+						substitute(ses, node->right, pto, flags_neol - SUB_VAR);
 
 						if (pti[1] == DEFAULT_OPEN)
 						{
@@ -206,9 +273,10 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 							{
 								pti += 2 + strlen(temp);
 
-								substitute(ses, temp, &tmp, SUB_VAR|SUB_FUN);
+								substitute(ses, temp, temp, SUB_VAR|SUB_FUN);
 
-								strcpy(pto, get_list_item(ses, node, tmp));
+								get_list_item(ses, node, temp);
+								strcpy(pto, temp);
 							}
 						}
 						pto += strlen(pto);
@@ -333,33 +401,33 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 						continue;
 					}
 
-					pti = get_arg_in_braces(&pti[i], &tmp, FALSE);
+					pti = get_arg_in_braces(&pti[i], temp, FALSE);
 
-					substitute(ses, tmp, &pte, flags_neol);
+					substitute(ses, temp, buf, flags_neol);
 
-					show_debug(ses, LIST_FUNCTION, "#FUNCTION DEBUG: {%s} {%s}", node->left, pte);
+					show_debug(ses, LIST_FUNCTION, "#FUNCTION DEBUG: {%s} {%s}", node->left, buf);
 
-					gtd->vars[0] = string_alloc(pte);
+					RESTRING(gtd->vars[0], buf);
 
-					for (i = 0 ; i < 10 ; i++)
+					pte = buf;
+
+					for (i = 1 ; i < 100 ; i++)
 					{
-						pte = get_arg_in_braces(pte, &gtd->cmds[i], FALSE);
+						pte = get_arg_in_braces(pte, temp, FALSE);
+
+						RESTRING(gtd->vars[i], temp);
+
+						if (*gtd->vars[i] == 0)
+						{
+							break;
+						}
 					}
 
-					for (i = 0 ; i < 9 ; i++)
-					{
-						gtd->vars[i+1] = gtd->cmds[i];
-					}
-
-					substitute(ses, node->right, &tmp, SUB_CMD|SUB_ARG);
-
-					parse_input(ses, tmp);
+					pre_parse_input(ses, node->right, SUB_ARG);
 
 					DEL_BIT(ses->flags, SES_FLAG_BREAK);
 
-					substitute(ses, "$result", &tmp, SUB_VAR);
-
-					strcpy(pto, tmp);
+					substitute(ses, "$result", pto, SUB_VAR);
 
 					show_debug(ses, LIST_FUNCTION, "#FUNCTION DEBUG: {%s} {%s}", node->left, pto);
 
@@ -372,54 +440,69 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 				break;
 
 			case '%':
-				if (HAS_BIT(flags, SUB_ARG) && isdigit(pti[1]))
+				if (HAS_BIT(flags, SUB_ARG) && (isdigit(pti[1]) || pti[1] == '%'))
 				{
-					ptt = gtd->vars[pti[1] - '0'];
-
-					while (*ptt)
+					if (pti[1] == '%')
 					{
-#ifdef BIG5
-						if (*ptt & 0x80)
+						while (pti[1] == '%')
 						{
-							*pto++ = *ptt++;
-							if (*ptt)
-							{
-								*pto++ = *ptt++;
-							}
-							continue;
+							*pto++ = *pti++;
 						}
-#endif
-						if (HAS_BIT(flags, SUB_SEC))
+						if (isdigit(pti[1]))
 						{
-							switch (*ptt)
-							{
-								case COMMAND_SEPARATOR:
-									*pto++ = '\\';
-									*pto++ = COMMAND_SEPARATOR;
-									break;
-
-								case '\\':
-									*pto++ = '\\';
-									*pto++ = '\\';
-									break;
-
-								default:
-									*pto++ = *ptt;
-									break;
-							}
-							ptt++;
+							pti++;
 						}
 						else
 						{
-							*pto++ = *ptt++;
+							*pto++ = *pti++;
 						}
 					}
-					pti += 2;
-				}
-				else if (HAS_BIT(flags, SUB_FMT))
-				{
-					*pto++ = '%';
-					*pto++ = *pti++;
+					else
+					{
+						i = isdigit(pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
+
+						ptt = gtd->vars[i];
+
+						while (*ptt)
+						{
+#ifdef BIG5
+							if (*ptt & 0x80)
+							{
+								*pto++ = *ptt++;
+								if (*ptt)
+								{
+									*pto++ = *ptt++;
+								}
+								continue;
+							}
+#endif
+							if (HAS_BIT(flags, SUB_SEC))
+							{
+								switch (*ptt)
+								{
+									case COMMAND_SEPARATOR:
+										*pto++ = '\\';
+										*pto++ = COMMAND_SEPARATOR;
+										break;
+
+									case '\\':
+										*pto++ = '\\';
+										*pto++ = '\\';
+										break;
+
+									default:
+										*pto++ = *ptt;
+										break;
+								}
+								ptt++;
+							}
+							else
+							{
+								*pto++ = *ptt++;
+							}
+						}
+						pti += isdigit(pti[2]) ? 3 : 2;
+					}
 				}
 				else
 				{
@@ -451,13 +534,33 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 				break;
 
 			case '&':
-				if (HAS_BIT(flags, SUB_CMD) && isdigit(pti[1]))
+				if (HAS_BIT(flags, SUB_CMD) && (isdigit(pti[1]) || pti[1] == '&'))
 				{
-					for (cnt = 0 ; gtd->cmds[pti[1] - '0'][cnt] ; cnt++)
+					if (pti[1] == '&')
 					{
-						*pto++ = gtd->cmds[pti[1] - '0'][cnt];
+						while (pti[1] == '&')
+						{
+							*pto++ = *pti++;
+						}
+						if (isdigit(pti[1]))
+						{
+							pti++;
+						}
+						else
+						{
+							*pto++ = *pti++;
+						}
+					}
+					else
+					{
+						i = isdigit(pti[2]) ? (pti[1] - '0') * 10 + pti[2] - '0' : pti[1] - '0';
+
+						for (cnt = 0 ; gtd->cmds[i][cnt] ; cnt++)
+						{
+							*pto++ = gtd->cmds[i][cnt];
+						}
+						pti += isdigit(pti[2]) ? 3 : 2;
 					}
-					pti += 2;
 				}
 				else
 				{
@@ -536,16 +639,10 @@ void substitute(struct session *ses, char *string, char **result, int flags)
 
 int check_one_action(char *line, char *original, char *action, struct session *ses)
 {
-	char *result, *ptr;
-	char *ptl;
-	int cnt;
+	char result[BUFFER_SIZE];
+	char *ptr, *ptl;
 
-	for (cnt = 0 ; cnt < 10 ; cnt++)
-	{
-		gtd->vars[cnt] = string_alloc("");
-	}
-
-	substitute(ses, action, &result, SUB_VAR|SUB_FUN|SUB_ESC);
+	substitute(ses, action, result, SUB_VAR|SUB_FUN|SUB_ESC);
 
 	ptr = result;
 
@@ -607,26 +704,23 @@ int action_regexp(char *exp, char *str, unsigned char arg)
 		switch (exp[0])
 		{
 			case '%':
-				if (exp[1] >= '0' && exp[1] <= '9')
+				if (isdigit(exp[1]))
 				{
-					arg = exp[1] - '0';
+					arg = isdigit(exp[2]) ? (exp[1] - '0') * 10 + exp[2] - '0' : exp[1] - '0';
 
-					if (exp[2] == 0)
+					if (exp[isdigit(exp[2]) ? 3 : 2] == 0)
 					{
-						gtd->vars[exp[1] - '0'] = string_realloc(gtd->vars[exp[1] - '0'], str);
+						RESTRING(gtd->vars[arg], str);
 
 						return TRUE;
 					}
 
 					for (cnt = 0 ; str[cnt] ; cnt++)
 					{
-						if (action_regexp(exp + 2, &str[cnt], arg))
+						if (action_regexp(exp + (isdigit(exp[2]) ? 3 : 2), &str[cnt], arg))
 						{
-							gtd->vars[exp[1] - '0'] = stringf_realloc(gtd->vars[exp[1] - '0'], "%.*s", cnt, str);
-/*
-							gtd->vars[exp[1] - '0'] = string_realloc(gtd->vars[exp[1] - '0'], str);
-							gtd->vars[exp[1] - '0'][cnt] = 0;
-*/
+							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt, str);
+
 							return TRUE;
 						}
 					}
@@ -651,7 +745,7 @@ int action_regexp(char *exp, char *str, unsigned char arg)
 					{
 						if (exp[cnt] == '|' || exp[cnt] == ']')
 						{
-							arg = (arg == 9) ? arg : arg + 1;
+							arg = (arg == 99) ? arg : arg + 1;
 							cnt--;
 							cnt2 = 0;
 
@@ -661,7 +755,7 @@ int action_regexp(char *exp, char *str, unsigned char arg)
 								cnt2++;
 								str++;
 							}
-							gtd->vars[arg] = stringf_alloc("%.*s", cnt2, &exp[cnt+1]);
+							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt2, &exp[cnt+1]);
 
 							while (*exp != ']')
 							{
