@@ -83,12 +83,12 @@
 */
 typedef struct listnode LNODE;
 
-typedef struct session *ARRAY   (struct session *ses, LNODE *list, char *arg);
-typedef void            CHAT    (char *arg);
+typedef struct session *ARRAY   (struct session *ses, LNODE *list, char *left, char *right);
+typedef void            CHAT    (char *left, char *right);
 typedef struct session *CLASS   (struct session *ses, char *left, char *right);
 typedef struct session *CONFIG  (struct session *ses, char *arg, int index);
 typedef struct session *COMMAND (struct session *ses, char *arg);
-typedef void            MAP     (struct session *ses, char *arg);
+typedef void            MAP     (struct session *ses, char *left, char *right);
 typedef void            CURSOR  (char *arg);
 typedef void            PATH    (struct session *ses, char *arg);
 
@@ -114,10 +114,10 @@ typedef void            PATH    (struct session *ses, char *arg);
 #define HISTORY_FILE         ".tt_history"
 
 #define STRING_SIZE                  50000
-#define BUFFER_SIZE                  10000
+#define BUFFER_SIZE                  20000
 #define NUMBER_SIZE                    100
 
-#define VERSION_NUM               "1.97.1"
+#define VERSION_NUM               "1.97.9"
 
 #define ESCAPE                          27
 
@@ -178,6 +178,12 @@ typedef void            PATH    (struct session *ses, char *arg);
 	Various flags
 */
 
+#define COL_BLD                       (1 << 1)
+#define COL_ITL                       (1 << 2)
+#define COL_UND                       (1 << 3)
+#define COL_BLK                       (1 << 4)
+#define COL_REV                       (1 << 5)
+
 #define CHAT_NAME_CHANGE                 1
 #define CHAT_REQUEST_CONNECTIONS         2
 #define CHAT_CONNECTION_LIST             3
@@ -234,13 +240,12 @@ typedef void            PATH    (struct session *ses, char *arg);
 #define SUB_ANC                       (1 <<  6)
 #define SUB_SEC                       (1 <<  7)
 #define SUB_EOL                       (1 <<  8)
-#define SUB_FMT                       (1 <<  9)
 
 
 #define TINTIN_FLAG_RESETBUFFER       (1 <<  0)
 #define TINTIN_FLAG_CONVERTMETACHAR   (1 <<  1)
 #define TINTIN_FLAG_HISTORYSEARCH     (1 <<  2)
-#define TINTIN_FLAG_QUIET             (1 <<  3)
+
 #define TINTIN_FLAG_USERCOMMAND       (1 <<  4)
 #define TINTIN_FLAG_INSERTINPUT       (1 <<  5)
 
@@ -260,10 +265,13 @@ typedef void            PATH    (struct session *ses, char *arg);
 #define SES_FLAG_LOGHTML              (1 << 17)
 #define SES_FLAG_GAG                  (1 << 18)
 #define SES_FLAG_UPDATEVTMAP          (1 << 19)
+#define SES_FLAG_COLORPATCH           (1 << 20)
 #define SES_FLAG_SCROLLLOCK           (1 << 21)
 #define SES_FLAG_SCAN                 (1 << 22)
 #define SES_FLAG_SCROLLSTOP           (1 << 23)
 #define SES_FLAG_CONVERTMETA          (1 << 24)
+#define SES_FLAG_RUN                  (1 << 25)
+
 #define SES_FLAG_BREAK                (1 << 28) /* wasn't the greatest idea */
 
 
@@ -432,12 +440,12 @@ typedef void            PATH    (struct session *ses, char *arg);
 
 #define SCROLL(ses)               (((ses)->cur_row >= (ses)->top_row && (ses)->cur_row <= (ses)->bot_row) || (ses)->cur_row == (ses)->rows)
 
-#define DO_ARRAY(array) struct session *array (struct session *ses, struct listnode *list, char *arg)
-#define DO_CHAT(chat) void chat (char *arg)
+#define DO_ARRAY(array) struct session *array (struct session *ses, struct listnode *list, char *left, char *right)
+#define DO_CHAT(chat) void chat (char *left, char *right)
 #define DO_CLASS(class) struct session *class (struct session *ses, char *left, char *right)
 #define DO_COMMAND(command) struct session  *command (struct session *ses, char *arg)
 #define DO_CONFIG(config) struct session *config (struct session *ses, char *arg, int index)
-#define DO_MAP(map) void map (struct session *ses, char *arg)
+#define DO_MAP(map) void map (struct session *ses, char *left, char *right)
 #define DO_PATH(path) void path (struct session *ses, char *arg)
 #define DO_CURSOR(cursor) void cursor (char *arg)
 
@@ -519,6 +527,7 @@ struct session
 	long long               connect_retry;
 	int                     connect_error;
 	char                    more_output[STRING_SIZE];
+	char                    color[100];
 	long long               check_output;
 };
 
@@ -528,15 +537,17 @@ struct tintin_data
 	struct session        * update;
 	struct chat_data      * chat;
 	struct memory_data    * mem;
-	struct termios          terminal;
+	struct termios          old_terminal;
+	struct termios          new_terminal;
 	char                  * mud_output_buf;
 	int                     mud_output_max;
 	int                     mud_output_len;
 	unsigned char         * mccp_buf;
 	int                     mccp_buf_max;
 	char                    input_buf[STRING_SIZE];
-	char                    input_tmp[BUFFER_SIZE];
-	char                    macro_buf[BUFFER_SIZE];
+	char                    input_tmp[STRING_SIZE];
+	char                    macro_buf[STRING_SIZE];
+	char                    paste_buf[STRING_SIZE];
 	int                     input_len;
 	int                     input_cur;
 	int                     input_pos;
@@ -550,11 +561,12 @@ struct tintin_data
 	int                     history_size;
 	int                     command_ref[26];
 	int                     flags;
+	int                     quiet;
 	char                    tintin_char;
 	char                    verbatim_char;
 	char                    repeat_char;
-	char                  * vars[10];
-	char                  * cmds[10];
+	char                  * vars[100];
+	char                  * cmds[100];
 };
 
 struct chat_data
@@ -591,12 +603,16 @@ struct array_type
 {
 	char                  * name;
 	ARRAY                 * array;
+	int                     lval;
+	int                     rval;
 };
 
 struct chat_type
 {
 	char                  * name;
 	CHAT                  * fun;
+	int                     lval;
+	int                     rval;
 	char                  * desc;
 };
 
@@ -647,6 +663,8 @@ struct map_type
 {
 	char                  * name;
 	MAP                   * map;
+	int                     lval;
+	int                     rval;
 };
 
 struct cursor_type
@@ -787,8 +805,8 @@ extern DO_ARRAY(array_len);
 extern DO_ARRAY(array_set);
 extern DO_ARRAY(array_srt);
 
-extern char *get_list_item(struct session *ses, struct listnode *node, char *arg);
-extern void  set_list_item(struct session *ses, struct listnode *node, char *left, char *right);
+extern void get_list_item(struct session *ses, struct listnode *node, char *arg);
+extern void set_list_item(struct session *ses, struct listnode *node, char *left, char *right);
 
 #endif
 
@@ -830,7 +848,6 @@ extern DO_CHAT(chat_who);
 extern DO_CHAT(chat_zap);
 
 extern  int chat_new(int s);
-extern void chat_call(char *ip);
 extern void close_chat(struct chat_data *buddy, int unlink);
 extern void process_chat_connections(fd_set *read_set, fd_set *write_set, fd_set *exc_set);
 extern void chat_forward_session(struct session *ses, char *linelog);
@@ -907,6 +924,7 @@ extern DO_MAP(map_delete);
 extern DO_MAP(map_destroy);
 extern DO_MAP(map_dig);
 extern DO_MAP(map_exit);
+extern DO_MAP(map_explore);
 extern DO_MAP(map_find);
 extern DO_MAP(map_flag);
 extern DO_MAP(map_get);
@@ -923,6 +941,7 @@ extern DO_MAP(map_name);
 extern DO_MAP(map_read);
 extern DO_MAP(map_roomflag);
 extern DO_MAP(map_set);
+extern DO_MAP(map_travel);
 extern DO_MAP(map_undo);
 extern DO_MAP(map_unlink);
 extern DO_MAP(map_walk);
@@ -945,13 +964,11 @@ extern void build_map_grid(short room, short x, short y, short z);
 extern void follow_map(struct session *ses, char *argument);
 extern void insert_undo(struct session *ses, char *format, ...);
 extern char *draw_room(struct session *ses, struct room_data *room, int line);
-
 extern void search_path(short room, short size);
 extern void shortest_path(struct session *ses, int walk, char *left, char *right);
-
+extern void explore_path(struct session *ses, int walk, char *left, char *right);
 extern int find_coord(struct session *ses, char *arg);
 extern void search_coord(int vnum, short x, short y, short z);
-
 extern void show_vtmap(struct session *ses);
 
 #endif
@@ -962,7 +979,7 @@ extern void show_vtmap(struct session *ses);
 extern DO_COMMAND(do_math);
 extern DO_COMMAND(do_if);
 extern double get_number(struct session *ses, char *str);
-extern void get_number_string(struct session *ses, char *str, char **result);
+extern void get_number_string(struct session *ses, char *str, char *result);
 extern double mathexp(struct session *ses, char *str);
 extern int mathexp_tokenize(struct session *ses, char *str);
 extern void mathexp_level(struct listnode *node);
@@ -990,11 +1007,13 @@ extern void do_one_prompt(struct session *ses, char *prompt, int row);
 #endif
 
 
-#ifndef __REGEXP_H__
-#define __REGEXP_H__
+#ifndef __TINEXP_H__
+#define __TINEXP_H__
 
 extern int regexp(char *exp, char *str, unsigned char cs);
-extern void substitute(struct session *ses, char *string, char **result, int flags);
+extern int regex(char *exp, char *str);
+DO_COMMAND(do_regexp);
+extern void substitute(struct session *ses, char *string, char *result, int flags);
 extern int check_one_action(char *line, char *original, char *action, struct session *ses);
 extern int action_regexp(char *exp, char *str, unsigned char arg);
 
@@ -1024,6 +1043,7 @@ extern DO_CONFIG(config_repeatchar);
 extern DO_CONFIG(config_debugtelnet);
 extern DO_CONFIG(config_convertmeta);
 extern DO_CONFIG(config_loglevel);
+extern DO_CONFIG(config_colorpatch);
 
 #endif
 
@@ -1072,6 +1092,7 @@ extern void buffer_f(char *arg);
 extern int push_call(char *f, ...);
 extern void pop_call(void);
 extern void dump_stack(void);
+extern void dump_full_stack(void);
 
 #endif
 
@@ -1099,7 +1120,7 @@ extern void check_all_actions(struct session *ses, char *original, char *line);
 
 extern struct session *do_alias(struct session *ses, char *arg);
 extern struct session *do_unalias(struct session *ses, char *arg);
-extern int check_all_aliases(struct session *ses, char *original, char *line, char **command);
+int                    check_all_aliases(struct session *ses, char *input);
 
 #endif
 
@@ -1109,10 +1130,8 @@ extern int check_all_aliases(struct session *ses, char *original, char *line, ch
 
 extern DO_COMMAND(do_read);
 extern DO_COMMAND(do_write);
-extern DO_COMMAND(do_session);
+
 extern void prepare_for_write(int mode, struct listnode *node, char *result);
-extern DO_COMMAND(do_textin);
-extern DO_COMMAND(do_scan);
 
 #endif 
 
@@ -1148,7 +1167,7 @@ extern DO_COMMAND(do_help);
 extern DO_COMMAND(do_highlight);
 extern int is_high_arg(char *s);
 extern DO_COMMAND(do_unhighlight);
-extern void check_all_highlights(struct session *ses, char **original, char *line);
+extern void check_all_highlights(struct session *ses, char *original, char *line);
 extern int get_highlight_codes(struct session *ses, char *htype, char *result);
 
 #endif
@@ -1228,16 +1247,11 @@ extern void quitmsg(char *message);
 #ifndef __MEMORY_H__
 #define __MEMORY_H__
 
-extern char *string_alloc(char *string);
-extern char *stringf_alloc(char *fmt, ...);
-extern char *string_realloc(char *point, char *string);
-extern char *stringf_realloc(char *point, char *fmt, ...);
-extern char *string_free(char *string);
-extern void memory_free(struct memory_data *mem);
+extern char *restring(char *point, char *string);
+extern char *refstring(char *point, char *fmt, ...);
 
 #endif
 
-
 #ifndef __MISC_H__
 #define __MISC_H__
 
@@ -1253,15 +1267,13 @@ extern DO_COMMAND(do_loop);
 extern DO_COMMAND(do_nop);
 extern DO_COMMAND(do_parse);
 extern DO_COMMAND(do_return);
-extern DO_COMMAND(do_run);
-extern DO_COMMAND(do_send);
+extern DO_COMMAND(do_run);extern DO_COMMAND(do_send);
 extern DO_COMMAND(do_showme);
 extern DO_COMMAND(do_snoop);
 extern DO_COMMAND(do_suspend);
-extern DO_COMMAND(do_system);
 extern DO_COMMAND(do_zap);
-
 extern DO_COMMAND(do_gagline);
+
 #endif
 
 
@@ -1270,7 +1282,7 @@ extern DO_COMMAND(do_gagline);
 
 extern int connect_mud(char *host, char *port, struct session *ses);
 extern void write_line_mud(char *line, struct session *ses);
-extern void read_buffer_mud(struct session *ses);
+extern int read_buffer_mud(struct session *ses);
 extern void readmud(struct session *ses);
 extern void process_mud_output(struct session *ses, char *linebuf, int prompt);
 
@@ -1317,20 +1329,20 @@ extern void init_telnet_session(struct session *ses);
 #ifndef __PARSE_H__
 #define __PARSE_H__
 
+extern struct session *pre_parse_input(struct session *ses, char *input, int flags);
 extern struct session *parse_input(struct session *ses, char *input);
-extern int is_speedwalk_dirs(char *cp);
-extern void process_speedwalk(char *cp, struct session *ses);
-extern struct session *parse_tintin_command(struct session *ses, char *command, char *arg);
-extern char *get_arg_all(char *string, char **result);
-extern char *get_arg_in_braces(char *string, char **result, int flag);
-extern char *get_arg_with_spaces(char *string, char **result);
-extern char *get_arg_stop_spaces(char *string, char **result);
-extern char *cpy_arg_in_braces(char *pti, char *arg, int flag);
-extern char *cpy_arg_with_spaces(char *pti, char *pto);
-extern char *cpy_arg_stop_spaces(char *pti, char *pto);
+extern struct session *parse_command(struct session *ses, char *input);
+
+extern int is_speedwalk(char *input);
+extern void process_speedwalk(struct session *ses, char *input);
+extern struct session *parse_tintin_command(struct session *ses, char *input);
+extern char *get_arg_all(char *string, char *result);
+extern char *get_arg_in_braces(char *string, char *result, int flag);
+extern char *get_arg_with_spaces(char *string, char *result);
+extern char *get_arg_stop_spaces(char *string, char *result);
 extern char *space_out(char *string);
-extern void write_mud(struct session *ses, char *command);
-extern void do_one_line(char **line, struct session *ses);
+extern void write_mud(struct session *ses, char *command, int flags);
+extern void do_one_line(char *line, struct session *ses);
 
 #endif
 
@@ -1344,6 +1356,7 @@ extern DO_PATH(path_ins);
 extern DO_PATH(path_load);
 extern DO_PATH(path_map);
 extern DO_PATH(path_new);
+extern DO_PATH(path_run);
 extern DO_PATH(path_save);
 extern DO_PATH(path_walk);
 
@@ -1366,27 +1379,39 @@ extern DO_COMMAND(do_untab);
 #ifndef __SESSION_H__
 #define __SESSION_H__
 
-
+extern DO_COMMAND(do_session);
 extern struct session *session_command(char *arg, struct session *ses);
 extern void show_session(struct session *ses, struct session *ptr);
 extern struct session *newactive_session(void);
 extern struct session *activate_session(struct session *ses);
-extern struct session *new_session(char *name, char *address, struct session *ses);
+extern struct session *new_session(struct session *ses, char *name, char *address, int desc);
 extern void connect_session(struct session *ses);
 extern void cleanup_session(struct session *ses);
 
-
 #endif
 
+
 #ifndef __SUBSTITUTE_H__
 #define __SUBSTITUTE_H__
 
 extern DO_COMMAND(do_substitute);
 extern DO_COMMAND(do_unsubstitute);
-extern void check_all_substitutions(struct session *ses, char **original, char *line);
+extern void check_all_substitutions(struct session *ses, char *original, char *line);
 
 #endif
 
+
+#ifndef __SYSTEM_H__
+#define __SYSTEM_H__
+
+extern DO_COMMAND(do_run);
+extern DO_COMMAND(do_scan);
+extern DO_COMMAND(do_script);
+extern DO_COMMAND(do_system);
+extern DO_COMMAND(do_textin);
+#endif
+
+
 #ifndef __TABLES_H__
 #define __TABLES_H__
 
@@ -1507,6 +1532,7 @@ extern int skip_vt102_codes_non_graph(char *str);
 extern void strip_vt102_codes(char *str, char *buf);
 extern void strip_vt102_codes_non_graph(char *str, char *buf);
 extern void strip_non_vt102_codes(char *str, char *buf);
+extern void get_color_codes(char *old, char *str, char *buf);
 extern int strip_vt102_strlen(char *str);
 extern int interpret_vt102_codes(struct session *ses, char *str, int real);
 
