@@ -35,6 +35,7 @@
 /* return: TRUE if s1 is an abbrevation of s2 */
 /**********************************************/
 
+
 int is_abbrev(char *s1, char *s2)
 {
 	if (strlen(s2) < strlen(s1))
@@ -342,7 +343,7 @@ void tintin_puts2(struct session *ses, char *string)
 		ses = gtd->ses;
 	}
 
-	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && HAS_BIT(gtd->flags, TINTIN_FLAG_QUIET))
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && gtd->quiet)
 	{
 		return;
 	}
@@ -389,7 +390,7 @@ void tintin_puts(struct session *ses, char *string)
 		ses = gtd->ses;
 	}
 
-	do_one_line(&string, ses);
+	do_one_line(string, ses);
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_GAG))
 	{
